#include	"compiler.h"

#include	"pc_core.h"
#include	"i286.h"
#include	"iobridge.h"
#include	"memory.h"
#include	"bios.h"
#include	"dosio.h"
#include	"dmac.h"
#include	"fdc.h"
#include	"fddfile.h"
#include	"fdd_mtr.h"
#include	"milstr.h"
#include	"nevent.h"
#include	"pic.h"
#include	"trace.h"

#define	CACHE_TABLES	4
#define	CACHE_BUFFER	32768

	char	hd_name[2][MAX_PATH];
	char	fdmode = 0;

enum {
	HDDTYPE_SASI,
	HDDTYPE_SCSI,
	HDDTYPE_NONE
};

typedef struct {
	char	vhd[3];
	char	ver[4];
	char	delimita;
	char	comment[128];
	DWORD	padding;
	BYTE	mbsize[2];					// little endian	x
	BYTE	sectorsize[2];				// little endian	x
	BYTE	sectors;
	BYTE	surfaces;
	BYTE	tracks[2];					// little endian	x
	BYTE	totals[4];					// little endian	x
} V98SCSI_TABLE;

typedef struct {
	BYTE	dummy[8];
	BYTE	headersize[4];				// little endian
	BYTE	filesize[4];				// little endian
	BYTE	sectorsize[4];				// little endian
	BYTE	sectors[4];					// little endian
	BYTE	surfaces[4];				// little endian
	BYTE	tracks[4];					// little endian
} HDI_HEADER;

typedef struct {
	long	totals;
	WORD	tracks;
	WORD	size;
	BYTE	sectors;
	BYTE	surfaces;
	WORD	type;
	DWORD	headersize;
	char	fname[MAX_PATH];
	FILEH	fh;
} SXSI_TABLE;


static SXSI_TABLE defsasi = {615*33*8, 615, 256, 33, 8,
											HDDTYPE_SASI, 256, {0x00}, (FILEH)-1};
										#if notyet
static SXSI_TABLE defscsi = {40*16*32*8, 40*16, 256, 32, 8,
											HDDTYPE_SCSI, 220, {0x00}, (FILEH)-1};
										#endif

SXSI_TABLE	sxsi_hd[4];

BYTE work[65536];

static	BYTE	mtr_c = 0;
static	DWORD	mtr_r = 0;									// ver0.26

void bios0x1b_init(void) {

	int		i;

	for (i=0; i<4; i++) {
		sxsi_hd[i].fh = (FILEH)-1;
	}
}

void bios0x1b_allclose(void) {

	int		i;

	for (i=0; i<4; i++) {
		if (sxsi_hd[i].fh != (FILEH)-1) {
			file_close(sxsi_hd[i].fh);
			sxsi_hd[i].fh = (FILEH)-1;
		}
	}
}

// -------------------------------------------------------------------- FDD

static void init_fdd_equip(void) {

	WORD	diskequip;

	diskequip = MEM_GETW(MEMW_DISK_EQUIP);
	diskequip &= 0x0f00;
	diskequip |= (WORD)(~fdmode) & 3;
	diskequip |= (WORD)fdmode << 12;
	MEM_SETW(MEMW_DISK_EQUIP, diskequip);
}

static void biosfd_setchrn(void) {

	fdc.C = I286_CL;
	fdc.H = I286_DH;
	fdc.R = I286_DL;
	fdc.N = I286_CH;
}

#if 0
static void biosfd_resultout(DWORD result) {

	BYTE	*p = mem + 0x00564 + (fdc.us*8);
	*p++ = (BYTE)(result & 0xff) | (fdc.hd << 2) | fdc.us;
	*p++ = (BYTE)(result >> 8);
	*p++ = (BYTE)(result >> 16);
	*p++ = fdc.C;
	*p++ = fdc.H;
	*p++ = fdc.R;
	*p++ = fdc.N;
	*p++ = fdc.ncn;
}
#endif

static BOOL biosfd_seek(BYTE track) {

	fdc.ncn = track; // * 2;
	mtr_c = track;
	if (fdd_seek()) {
		return(FAILURE);
	}
	return(SUCCESS);
}

static WORD fdfmt_biospara(BYTE fmt) {

	WORD	seg, off;
	WORD	n;

	n = fdc.N;
	if (n >= 4) {
		n = 3;
	}
	if (CTRL_FDMEDIA == DISKTYPE_2HD) {
		seg = MEM_GETW(MEMW_F2HD_P_SEG);
		off = MEM_GETW(MEMW_F2HD_P_OFF);
	}
	else {
		seg = MEM_GETW(MEMW_F2DD_P_SEG);
		off = MEM_GETW(MEMW_F2DD_P_OFF);
	}
	off += fdc.us * 2;
	off = i286_memword_read(seg, off);
	off += n * 8;
	if (!(I286_AH & 0x40)) {
		off += 4;
	}
	if (fmt) {
		off += 2;
	}
	return(i286_memword_read(seg, off));
}



enum {
	FDCBIOS_NORESULT,
	FDCBIOS_SUCCESS,
	FDCBIOS_SEEKSUCCESS,
	FDCBIOS_ERROR,
	FDCBIOS_SEEKERROR,
	FDCBIOS_READERROR,
	FDCBIOS_WRITEERROR,
	FDCBIOS_NONREADY,
	FDCBIOS_WRITEPROTECT
};

#if 1
static void fdd_int(int result) {

	if ((result == FDCBIOS_NORESULT) || (CTRL_FDMEDIA == DISKTYPE_2D)) {
		return;
	}
	switch(I286_AH & 0x0f) {
		case 0x00:								// 
		case 0x01:								// ٥ե
		case 0x02:								// Ǥΰ٤ɤ߹
		case 0x05:								// ǡν񤭹
		case 0x06:								// ǡɤ߹
//		case 0x07:								// إ
//		case 0x0a:								// READ ID
		case 0x0d:								// եޥå
			break;
		default:
			return;
	}
	fdc.stat[fdc.us] = (fdc.hd << 2) | fdc.us;
	switch(result) {
		case FDCBIOS_SUCCESS:
			fdcsend_success7();
			break;
		case FDCBIOS_SEEKSUCCESS:
		case FDCBIOS_SEEKERROR:
			fdc.stat[fdc.us] |= FDCRLT_SE;
			fdc_interrupt();
			fdc.event = FDCEVENT_NEUTRAL;
			fdc.status = FDCSTAT_RQM;
			break;
		case FDCBIOS_READERROR:
			fdc.stat[fdc.us] |= FDCRLT_IC0 | FDCRLT_ND;
			fdcsend_error7();
			break;
		case FDCBIOS_WRITEERROR:
			fdc.stat[fdc.us] |= FDCRLT_IC0 | FDCRLT_EN;
			fdcsend_error7();
			break;
		case FDCBIOS_NONREADY:
			fdc.stat[fdc.us] |= FDCRLT_IC0 | FDCRLT_NR;
			fdcsend_error7();
			break;
		case FDCBIOS_WRITEPROTECT:
			fdc.stat[fdc.us] |= FDCRLT_IC0 | FDCRLT_NW;
			fdcsend_error7();
			break;
	}
}
#endif

static BYTE fdd_operate(BYTE type) {

	BYTE	ret_ah = 0x60;
	WORD	size;
	WORD	pos;
	WORD	accesssize;
	WORD	secsize;
	WORD	para;
	BYTE	s;
	BYTE	ID[4];
	BYTE	hd;
	int		result = FDCBIOS_NORESULT;
	DWORD	addr;

	mtr_c = 0xff;
	mtr_r = 0;

//	TRACE_("int 1Bh", I286_AH);

	if ((I286_AH & 0x0f) != 0x0a) {
		fdc.crcn = 0;
	}
	if ((I286_AH & 0x0f) != 0x03) {
		CTRL_FDMEDIA = type;
		switch(type) {
			case DISKTYPE_2HD:
				if (pic[1].imr & PIC_INT42) {
					return(0xd0);
				}
				break;
			case DISKTYPE_2DD:
				if (pic[1].imr & PIC_INT41) {
					return(0xd0);
				}
				break;
		}
		if (fdc.us != (I286_AL & 0x03)) {
			fdc.us = I286_AL & 0x03;
			fdc.crcn = 0;
		}
		hd = ((I286_DH) ^ (I286_AL >> 2)) & 1;
		if (fdc.hd != hd) {
			fdc.hd = hd;
			fdc.crcn = 0;
		}
		if (!fdd_diskready(fdc.us)) {
			fdd_int(FDCBIOS_NONREADY);
			if (I286_AH == 0x84) {				// ver0.28
				return(0x68);			// 󥹤 ξѥɥ饤־
			}
			return(0x60);
		}
	}

	switch(I286_AH & 0x0f) {
		case 0x00:								// 
			if (I286_AH & 0x10) {
				if (!biosfd_seek(I286_CL)) {
					result = FDCBIOS_SEEKSUCCESS;
				}
				else {
					result = FDCBIOS_SEEKERROR;
				}
			}
			ret_ah = 0x00;
			break;

		case 0x01:								// ٥ե
			if (I286_AH & 0x10) {
				if (!biosfd_seek(I286_CL)) {
					result = FDCBIOS_SEEKSUCCESS;
				}
				else {
					ret_ah = 0xe0;
					result = FDCBIOS_SEEKERROR;
					break;
				}
			}
			biosfd_setchrn();
			para = fdfmt_biospara(0);
			if (!para) {
				ret_ah = 0xd0;
				break;
			}
			if (fdc.N < 8) {
				secsize = 128 << fdc.N;
			}
			else {
				secsize = 128 << 8;
			}
			size = I286_BX;
			while(size) {
				if (size > secsize) {
					accesssize = secsize;
				}
				else {
					accesssize = size;
				}
				if (fdd_read()) {
					break;
				}
				size -= accesssize;
				mtr_r += accesssize;						// ver0.26
				if ((fdc.R++ == (BYTE)para) && (I286_AH & 0x80) && (!fdc.hd)) {
					fdc.hd = 1;
					fdc.H = 1;
					fdc.R = 1;
					if (biosfd_seek(fdc.treg[fdc.us])) {
						break;
					}
				}
			}
			if (!size) {
				ret_ah = 0x00;
				result = FDCBIOS_SUCCESS;
			}
			else {
				ret_ah = 0xc0;
				result = FDCBIOS_READERROR;
			}
			break;

		case 0x03:								// 
			init_fdd_equip();
			ret_ah = 0x00;
			break;

		case 0x04:								// 
#if 0											// ver0.28
			if (fdd_diskaccess()) {
				break;
			}
#endif
			ret_ah = 0x01;
			if (fdd_diskprotect(fdc.us)) {
				ret_ah |= 0x10;
			}
			if (type == DISKTYPE_2DD) {
				ret_ah |= 4;
			}
			if (I286_AH == 0x84) {				// ver0.28
				ret_ah |= 8;					// 1MB/640KBξѥɥ饤
			}
			break;

		case 0x05:								// ǡν񤭹
			if (I286_AH & 0x10) {
				if (!biosfd_seek(I286_CL)) {
					result = FDCBIOS_SEEKSUCCESS;
				}
				else {
					ret_ah = 0xe0;
					result = FDCBIOS_SEEKERROR;
					break;
				}
			}
			biosfd_setchrn();
			para = fdfmt_biospara(0);
			if (!para) {
				ret_ah = 0xd0;
				break;
			}
			if (fdd_diskprotect(fdc.us)) {
				ret_ah = 0x70;
				result = FDCBIOS_WRITEPROTECT;
				break;
			}
			if (fdc.N < 8) {
				secsize = 128 << fdc.N;
			}
			else {
				secsize = 128 << 8;
			}
			size = I286_BX;
			addr = ES_BASE + I286_BP;
			while(size) {
				if (size > secsize) {
					accesssize = secsize;
				}
				else {
					accesssize = size;
				}
				i286_memx_read(addr, fdc.buf, accesssize);
				if (fdd_write()) {
					break;
				}
				addr += accesssize;
				size -= accesssize;
				mtr_r += accesssize;						// ver0.26
				if ((fdc.R++ == (BYTE)para) && (I286_AH & 0x80) && (!fdc.hd)) {
					fdc.hd = 1;
					fdc.H = 1;
					fdc.R = 1;
					if (biosfd_seek(fdc.treg[fdc.us])) {
						break;
					}
				}
			}
			if (!size) {
				ret_ah = 0x00;
				result = FDCBIOS_SUCCESS;
			}
			else {
				ret_ah = fddlasterror;			// 0xc0		// ver0.28
				result = FDCBIOS_WRITEERROR;
			}
			break;

		case 0x02:								// Ǥΰ٤ɤ߹
		case 0x06:								// ǡɤ߹
			if (I286_AH & 0x10) {
				if (!biosfd_seek(I286_CL)) {
					result = FDCBIOS_SEEKSUCCESS;
				}
				else {
					ret_ah = 0xe0;
					result = FDCBIOS_SEEKERROR;
					break;
				}
			}
			biosfd_setchrn();
			para = fdfmt_biospara(0);
			if (!para) {
				ret_ah = 0xd0;
				break;
			}
#if 0
			if (fdc.R >= 0xf5) {
				ret_ah = 0xb0;
				break;
			}
#endif
			if (fdc.N < 8) {
				secsize = 128 << fdc.N;
			}
			else {
				secsize = 128 << 8;
			}
			size = I286_BX;
			addr = ES_BASE + I286_BP;
			while(size) {
				if (size > secsize) {
					accesssize = secsize;
				}
				else {
					accesssize = size;
				}
				if (fdd_read()) {
					break;
				}
				i286_memx_write(addr, fdc.buf, accesssize);
				addr += accesssize;
				size -= accesssize;
				mtr_r += accesssize;						// ver0.26
				if ((fdc.R++ == (BYTE)para) && (I286_AH & 0x80) && (!fdc.hd)) {
					fdc.hd = 1;
					fdc.H = 1;
					fdc.R = 1;
					if (biosfd_seek(fdc.treg[fdc.us])) {
						break;
					}
				}
			}
			if (!size) {
				ret_ah = fddlasterror;				// 0x00;
				result = FDCBIOS_SUCCESS;
			}
#if 1
			else if ((I286_AH & 0x0f) == 0x02) {	// ARSк
				ret_ah = 0x00;
				result = FDCBIOS_READERROR;
			}
#endif
			else {
				ret_ah = fddlasterror;				// 0xc0;	// ver0.28
				result = FDCBIOS_READERROR;
			}
			break;

		case 0x07:						// إ
			biosfd_seek(0);
			ret_ah = 0x00;
			result = FDCBIOS_SEEKSUCCESS;
			break;

		case 0x0a:						// READ ID
			if (I286_AH & 0x10) {							// ver0.28
				if (!biosfd_seek(I286_CL)) {
					result = FDCBIOS_SEEKSUCCESS;
				}
				else {
					ret_ah = 0xe0;
					result = FDCBIOS_SEEKERROR;
					break;
				}
			}
			if (fdd_readid()) {
				ret_ah = fddlasterror;			// 0xa0;	// ver0.28
				break;
			}
			if (fdc.N < 8) {								// ver0.26
				mtr_r += 128 << fdc.N;
			}
			else {
				mtr_r += 128 << 8;
			}
			ret_ah = 0x00;
			I286_CL = fdc.C;
			I286_DH = fdc.H;
			I286_DL = fdc.R;
			I286_CH = fdc.N;
			break;

		case 0x0d:						// եޥå
			if (I286_AH & 0x10) {
				biosfd_seek(I286_CL);
			}
			if (fdd_diskprotect(fdc.us)) {
				ret_ah = 0x70;
				break;
			}
			fdc.d = I286_DL;
			fdc.N = I286_CH;
			para = fdfmt_biospara(1);
			if (!para) {
				ret_ah = 0xd0;
				break;
			}
			fdc.sc = (BYTE)para;
			fdd_formatinit();
			pos = I286_BP;
			for (s=0; s<fdc.sc; s++) {
				i286_memstr_read(I286_ES, pos, ID, 4);
				fdd_formating(ID);
				pos += 4;
			}
			ret_ah = 0x00;
			break;
	}
	fdd_int(result);
	fddmtr_seek(fdc.us, mtr_c, mtr_r);
	I286_IP = BIOS_OFF + 0x40;										// ver0.29
	return(ret_ah);
}


// -------------------------------------------------------------------- SASI

static void init_sasi_equip(void) {

	BYTE	i;
	WORD	bit;
	WORD	diskequip;

	diskequip = MEM_GETW(MEMW_DISK_EQUIP);
	diskequip &= 0xf0ff;
	for (i=0, bit=0x0100; i<2; i++, bit<<=1) {
		if (sxsi_hd[i].fname[0]) {
			diskequip |= bit;
		}
	}
	MEM_SETW(MEMW_DISK_EQUIP, diskequip);
}

static void init_scsi_equip(void) {

	BYTE	i;
	BYTE	bit;
	WORD	w;
	BYTE	diskequips;

	diskequips = 0;
	ZeroMemory(mem + 0x00460, 0x20);

	for (i=0, bit=1; i<2; i++, bit<<=1) {
		if (sxsi_hd[i+2].fname[0]) {
			diskequips |= bit;
			switch(sxsi_hd[i+2].size) {
				case 256:
					w = 0 << 12;
					break;
				case 512:
					w = 1 << 12;
					break;
				default:
					w = 2 << 12;
					break;
			}
			MEM_SETB(0x00460+i*4, sxsi_hd[i+2].sectors);
			MEM_SETB(0x00461+i*4, sxsi_hd[i+2].surfaces);
			MEM_SETW(0x00462+i*4, (w | sxsi_hd[i+2].tracks));
		}
	}
	MEM_SETB(MEMB_DISK_EQUIPS, diskequips);
}


static BYTE sxsi_pos(long *pos) {

	SXSI_TABLE	*sxsi;
	int			np2drv;

	*pos = 0;
	np2drv = (I286_AL & 0x20) >> 4;
	if ((I286_AL & 0x0f) >= 2) {
		return(0x60);
	}
	np2drv |= (I286_AL & 1);
	sxsi = &sxsi_hd[np2drv];
	if (I286_AL & 0x80) {
		if ((I286_DL >= sxsi->sectors) ||
			(I286_DH >= sxsi->surfaces) ||
			(I286_CX >= sxsi->tracks)) {
			return(0xd0);
		}
		(*pos) = ((I286_CX * sxsi->surfaces) + I286_DH) * sxsi->sectors
															+ I286_DL;
	}
	else {
		*pos = (I286_DL << 16) | I286_CX;
		if (!(I286_AL & 0x20)) {
			(*pos) &= 0x1fffff;
		}
		if ((*pos) >= sxsi->totals) {
			return(0xd0);
		}
	}
	return(0x00);
}

#if 0
typedef struct {
	BYTE	drv;
	BYTE	pry;
	BYTE	write;
	BYTE	dummys;
	long	basesector;
	long	sectors;
} CACHE_TABLE;

static	CACHE_TABLE	sxsicache[CACHE_TABLES];
static	BYTE		sxsibuf[CACHE_TABLES][CACHE_BUFFER];

static void sxsicache_init(void) {

	int		i;

	for (i=0; i<CACHE_TABLES; i++) {
		sxsicache[i].drv = 0xff;
		sxsicache[i].pry = (BYTE)i;
	}
}

static void trush_cache(BYTE num) {

	FILEH	hdl;
	BYTE	np2drv;
	long	pos;
	WORD	size;

	while(sxsicache[num].write) {
		sxsicache[num].write = 0;
		np2drv = sxsicache[num].drv;
		if ((np2drv == 0xff) ||
			(!sxsi_hd[np2drv].fname[0]) ||
			((hdl = file_open(sxsi_hd[np2drv].fname)) == (FILEH)-1)) {
			break;
		}
		pos = sxsicache[num].basesector * sxsi_hd[np2drv].size
											+ sxsi_hd[np2drv].headersize;
		size = (WORD)(sxsicache[num].sectors * sxsi_hd[np2drv].size);
		if ((size) && (file_seek(hdl, pos, FSEEK_SET) == (DWORD)pos)) {
			file_write(hdl, sxsibuf[num], size);
		}
		file_close(hdl);
	}
	sxsicache[num].drv = 0xff;
}


static BYTE sxsicache_set(BYTE np2drv, long pos) {

	DWORD		i, j;
	SXSI_TABLE	*sxsi = &sxsi_hd[np2drv];
	CACHE_TABLE	*cache;
	FILEH		hdl;
	long		fpos;
	long		sec;
	WORD		size;

	if (pos >= sxsi->totals) {
		return(-1);
	}

	// å夵Ƥ뤫
	for (i=0, cache=sxsicache; i<CACHE_TABLES; i++, cache++) {
		if ((cache->drv == np2drv) &&
			(cache->basesector >= pos) &&
			((cache->basesector + cache->sectors) < pos)) {
			for (j=0; j<CACHE_TABLES; j++) {
				sxsicache[j].pry++;
			}
			cache->pry = 0;
			return((BYTE)i);
		}
	}

	// ֻȤƤʤåõ
	i = 0;
	for (j=1; j<CACHE_TABLES; j++) {
		if (sxsicache[i].pry < sxsicache[j].pry) {
			i = j;
		}
	}
	trush_cache((BYTE)i);

	// åɤ߹
	if (!sxsi->size) {
		return(-1);
	}
	sec = CACHE_BUFFER / (sxsi->size);
	if ((pos + sec) > sxsi->totals) {
		sec = sxsi->totals - pos;
	}
	// ˰㤦ϥɥǥå夵Ƥ뤫
	for (j=0, cache=sxsicache; j<CACHE_TABLES; j++, cache++) {
		if ((cache->drv == np2drv) &&
			(cache->basesector < pos) &&
			(cache->basesector > (pos + sec))) {
			sec = cache->basesector - pos;
		}
	}
	fpos = pos * sxsi_hd[np2drv].size + sxsi_hd[np2drv].headersize;
	size = sec * sxsi_hd[np2drv].size;
	if (!size) {
		return(-1);
	}
	if ((!sxsi_hd[np2drv].fname[0]) ||
		((hdl = file_open(sxsi_hd[np2drv].fname)) == (FILEH)-1)) {
		return(-1);
	}
	if ((file_seek(hdl, pos, FSEEK_SET) != (DWORD)pos) ||
		(file_read(hdl, sxsibuf[i], size) == -1)) {
		file_close(hdl);
		return(-1);
	}
	file_close(hdl);
	for (j=0; j<CACHE_TABLES; j++) {
		sxsicache[j].pry++;
	}
	sxsicache[i].drv = np2drv;
	sxsicache[i].pry = 0;
	sxsicache[i].write = 0;
	sxsicache[i].basesector = pos;
	sxsicache[i].sectors = sec;
	return((BYTE)i);
}
#endif



static BYTE sxsi_read(BYTE drv, long pos, BYTE *dest, WORD size) {

	FILEH	hdh;
	long	r;
	WORD	rsize;
	int		np2drv;

	np2drv = (drv & 0x20) >> 4;
	if ((drv & 0x0f) >= 2) {
		return(0x60);
	}
	np2drv |= (drv & 1);
	if (!sxsi_hd[np2drv].fname[0]) {
		return(0x60);
	}
	hdh = sxsi_hd[np2drv].fh;
	pos = pos * sxsi_hd[np2drv].size + sxsi_hd[np2drv].headersize;
	r = file_seek(hdh, pos, FSEEK_SET);
	if (r == -1) {
		return(0x40);
	}
	if (pos != r) {
		return(0xd0);
	}
	while(size) {
		if (size >= sxsi_hd[np2drv].size) {
			rsize = sxsi_hd[np2drv].size;
		}
		else {
			rsize = size;
		}
		nevent.remainclock -= rsize;
		if (file_read(hdh, dest, rsize) != rsize) {
			return(0xd0);
		}
		dest += rsize;
		size -= rsize;
	}
	return(0x00);
}


static BYTE sxsi_write(BYTE drv, long pos, BYTE *src, WORD size) {

	FILEH	hdh;
	long	r;
	WORD	wsize;
	int		np2drv;

	np2drv = (drv & 0x20) >> 4;
	if ((drv & 0x0f) >= 2) {
		return(0x60);
	}
	np2drv |= (drv & 1);
	if (!sxsi_hd[np2drv].fname[0]) {
		return(0x60);
	}
	hdh = sxsi_hd[np2drv].fh;
	pos = pos * sxsi_hd[np2drv].size + sxsi_hd[np2drv].headersize;
	r = file_seek(hdh, pos, FSEEK_SET);
	if (r == -1) {
		return(0x40);
	}
	if (pos != r) {
		return(0xd0);
	}
	while(size) {
		if (size >= sxsi_hd[np2drv].size) {
			wsize = sxsi_hd[np2drv].size;
		}
		else {
			wsize = size;
		}
		nevent.remainclock -= wsize;
		if (file_write(hdh, src, wsize) != wsize) {
			return(0x70);
		}
		src += wsize;
		size -= wsize;
	}
	return(0x00);
}

static BYTE sxsi_format(BYTE drv, long pos) {

	FILEH	hdh;
	long	r;
	int		np2drv;
	WORD	i;

	np2drv = (drv & 0x20) >> 4;
	if ((drv & 0x0f) >= 2) {
		return(0x60);
	}
	np2drv |= (drv & 1);
	if (!sxsi_hd[np2drv].fname[0]) {
		return(0x60);
	}
	hdh = sxsi_hd[np2drv].fh;
	pos = pos * sxsi_hd[np2drv].size + sxsi_hd[np2drv].headersize;
	r = file_seek(hdh, pos, FSEEK_SET);
	if (r == -1) {
		return(0x40);
	}
	if (pos != r) {
		return(0xd0);
	}
	memset(work, 0xe5, sxsi_hd[np2drv].size);
	for (i=0; i<sxsi_hd[np2drv].sectors; i++) {
		nevent.remainclock -= sxsi_hd[np2drv].size;
		if (file_write(hdh, work, sxsi_hd[np2drv].size)
												!= sxsi_hd[np2drv].size) {
			return(0x70);
		}
	}
	return(0x00);
}

static BYTE sxsi_operate(BYTE type) {

	BYTE	ret_ah = 0x00;
	BYTE	drv;
	long	pos;
//	int		i;

	drv = (I286_AL & 0x20) >> 4;
	if ((I286_AL & 0x0f) >= 2) {
		return(0x60);
	}
	drv |= (I286_AL & 1);

	switch(I286_AH & 0x0f) {
		case 0x01:						// ٥ե
		case 0x07:						// ȥ饯
		case 0x0f:						// ȥ饯
			break;

   		case 0x03:						// ˥饤
			if (type == HDDTYPE_SASI) {
				init_sasi_equip();
			}
			else if (type == HDDTYPE_SCSI) {
				init_scsi_equip();
			}
			break;

   		case 0x04:						// 
			ret_ah = 0x00;
			if ((I286_AH == 0x04) && (type == HDDTYPE_SASI)) {
				ret_ah = 0x04;
			}
			else if ((I286_AH == 0x44) && (type == HDDTYPE_SCSI)) {
				I286_BX = 1;
			}
			else if (I286_AH == 0x84) {
				I286_BX = sxsi_hd[drv].size;
				I286_CX = sxsi_hd[drv].tracks;
				I286_DH = sxsi_hd[drv].surfaces;
				I286_DL = sxsi_hd[drv].sectors;
			}
			break;

		case 0x05:						// ǡν񤭹
			i286_memx_read(ES_BASE + I286_BP, work, I286_BX);
			ret_ah = sxsi_pos(&pos);
			if (!ret_ah) {
				ret_ah = sxsi_write(I286_AL, pos, work, I286_BX);
			}
			break;

		case 0x06:						// ǡɤ߹
			ret_ah = sxsi_pos(&pos);
			if (!ret_ah) {
				ret_ah = sxsi_read(I286_AL, pos, work, I286_BX);
				if (ret_ah < 0x20) {
					i286_memx_write(ES_BASE + I286_BP, work, I286_BX);
				}
			}
			break;

		case 0x0d:						// եޥå
			if (I286_DL) {
				ret_ah = 0x30;
				break;
			}
			i286_memstr_read(I286_ES, I286_BP, work, I286_BX);
			ret_ah = sxsi_pos(&pos);
#if 1
			if (!ret_ah) {
				ret_ah = sxsi_format(I286_AL, pos);
			}
#else
			memset(work, 0xe5, sxsi_hd[drv].size);
			for (i=0; (i<sxsi_hd[drv].sectors) && (!ret_ah); i++) {
				ret_ah = sxsi_write(I286_AL, pos++, work, sxsi_hd[drv].size);
			}
#endif
			break;

		default:
			ret_ah = 0x40;
			break;
	}
	return(ret_ah);
}

// -------------------------------------------------------------------- BIOS

void bios0x1b_hddreopen(void) {

	int				i;
	FILEH			hdh;
	int				ftype;
	char			*ext;
	SXSI_TABLE		*sxsi;
	int				sasi = 0;
#if notyet
	int				scsi = 0;
#endif
	HDI_HEADER		hdi;
//	V98SCSI_TABLE	v98;

	bios0x1b_allclose();
	ZeroMemory(sxsi_hd, sizeof(sxsi_hd));
	bios0x1b_init();

	for (i=0; i<2; i++) {
		if (hd_name[i][0]) {
			ftype = file_getftype(hd_name[i]);
			if ((ftype != FTYPE_THD) && (ftype != FTYPE_HDI)) {
				ext = getExtName(hd_name[i]);
				if (!milstr_cmpA(ext, "THD")) {			// T98 HDD (SASI)
					ftype = FTYPE_THD;
				}
				else if (!milstr_cmpA(ext, "HDI")) {	// ANEX86 HDD (SASI)
					ftype = FTYPE_HDI;
				}
			}
			if (ftype == FTYPE_THD) {					// T98 HDD (SASI)
				hdh = file_open(hd_name[i]);
				if (hdh != (FILEH)-1) {
					BYTE tracks[2];
					sxsi = &sxsi_hd[sasi];
					*sxsi = defsasi;
					if (file_read(hdh, &tracks, 2) == 2) {
						sxsi->tracks = GETWORD(tracks);
						sxsi->totals = sxsi->tracks *
											sxsi->sectors * sxsi->surfaces;
						strcpy(sxsi->fname, hd_name[i]);
						sxsi->fh = hdh;
						sasi++;
					}
					else {
						file_close(hdh);
					}
				}
			}
			else if (ftype == FTYPE_HDI) {			// T98 HDD (SASI)
				hdh = file_open(hd_name[i]);		// thanx Mamiya
				if (hdh != (FILEH)-1) {
					sxsi = &sxsi_hd[sasi];
					if (file_read(hdh, &hdi, sizeof(hdi)) == sizeof(hdi)) {
						sxsi->size = GETWORD(hdi.sectorsize);
						sxsi->headersize = GETDWORD(hdi.headersize);
						sxsi->tracks = GETWORD(hdi.tracks);
						sxsi->surfaces = hdi.surfaces[0];
						sxsi->sectors = hdi.sectors[0];
						sxsi->totals = sxsi->tracks *
											sxsi->sectors * sxsi->surfaces;
						strcpy(sxsi->fname, hd_name[i]);
						sxsi->fh = hdh;
						sasi++;
					}
					else {
						file_close(hdh);
					}
				}
			}
#if 0
			else if (!milstr_cmpA(ext, "HDD")) {	// Virtual98 HDD (SCSI)
				hdh = file_open(hd_name[i]);
				if (hdh != (FILEH)-1) {
					if ((file_read(hdh, &v98, sizeof(v98)) == sizeof(v98)) &&
						(!memcmp(v98.vhd, "VHD", 3))) {
						sxsi = &sxsi_hd[scsi+2];
						*sxsi = defscsi;
						sxsi->totals = v98.totals;
						sxsi->tracks = v98.tracks;
						sxsi->size = v98.sectorsize;
						sxsi->sectors = v98.sectors;
						sxsi->surfaces = v98.surfaces;
						strcpy(sxsi->fname, hd_name[i]);
						sxsi->fh = hdh;
						scsi++;
					}
					else {
						file_close(hdh);
					}
				}
			}
#endif
		}
	}
}


static WORD boot_fd1(void) {									// ver0.27

	WORD	remain;
	WORD	size;
	DWORD	pos;
	WORD	bootseg;

	if (biosfd_seek(0)) {
		return(0);
	}
	fdc.hd = 0;
	if (fdd_readid()) {
		return(0);
	}
	fdc.R = 1;
	remain = 0x400;
	pos = 0x1fc00;
	if (!fdc.N) {
		ZeroMemory(mem + 0x1fc00, 0x200);
		pos += 0x200;
		remain -= 0x200;
	}
	bootseg = (WORD)(pos >> 4);
	while(remain) {
		if (fdd_read()) {
			return(0);
		}
		if (fdc.N < 3) {
			size = 128 << fdc.N;
		}
		else {
			size = 128 << 3;
		}
		if (remain < size) {
			CopyMemory(mem + pos, fdc.buf, remain);
			break;
		}
		else {
			CopyMemory(mem + pos, fdc.buf, size);
			pos += size;
			remain -= size;
			fdc.R++;
		}
	}
	return(bootseg);
}

static WORD boot_fd(BYTE drv, BYTE type) {						// ver0.27

	WORD	bootseg;

	if (drv >= 4) {
		return(0);
	}
	fdc.us = drv & 3;
	if (!fdd_diskready(fdc.us)) {
		return(0);
	}

	// 2HD
	if (type & 1) {
		CTRL_FDMEDIA = DISKTYPE_2HD;
		bootseg = boot_fd1();
		if (bootseg) {
			MEM_SETB(MEMB_DISK_BOOT, (BYTE)(0x90+drv));
			return(bootseg);
		}
	}
	if (type & 2) {
		// 2D
		CTRL_FDMEDIA = DISKTYPE_2DD;
		fddfile[fdc.us].mode2d = 1;
		bootseg = boot_fd1();
		if (bootseg) {
			MEM_SETB(MEMB_DISK_BOOT, (BYTE)(0x70+drv));
			fdmode = 3;
			return(bootseg);
		}

		// 2DD
		fddfile[fdc.us].mode2d = 0;
		bootseg = boot_fd1();
		if (bootseg) {
			MEM_SETB(MEMB_DISK_BOOT, (BYTE)(0x10+drv));
			fdmode = 3;
			return(bootseg);
		}
	}
	return(0);
}


WORD boot_hd(BYTE drv) {										// ver0.27

	BYTE	ret;

	ret = sxsi_read(drv, 0, mem + 0x1fc00, 0x400);
	if (ret < 0x20) {
		MEM_SETB(MEMB_DISK_BOOT, drv);
		return(0x1fc0);
	}
	return(0);
}


WORD bootstrapload(void) {										// ver0.27

	BYTE	i;
	WORD	bootseg;

	fdmode = 0;
	bootseg = 0;
									// AL쥸ͤ
	switch(MEM_GETB(MEMB_MSW5) & 0xf0) {
		case 0x00:					// Ρޥ
			break;
		case 0x20:					// 640KB FDD
			for (i=0; (i<4) && (!bootseg); i++) {
				if (fdd_diskready(i)) bootseg = boot_fd(i, 2);
			}
			break;
		case 0x40:					// 1.2MB FDD
			for (i=0; (i<4) && (!bootseg); i++) {
				if (fdd_diskready(i)) bootseg = boot_fd(i, 1);
			}
			break;
		case 0x60:					// MO
			break;
		case 0xa0:					// SASI 1
			if (sxsi_hd[0].fname[0]) bootseg = boot_hd(0x80);
			break;
		case 0xb0:					// SASI 2
			if (sxsi_hd[1].fname[0]) bootseg = boot_hd(0x81);
			break;
		case 0xc0:					// SCSI
			for (i=0; (i<2) && (!bootseg); i++) {
				if (sxsi_hd[i+2].fname[0]) bootseg = boot_hd(i | 0xa0);
			}
			break;
		default:					// ROM
			return(0);
	}
	for (i=0; (i<4) && (!bootseg); i++) {
		if (fdd_diskready(i)) {
			bootseg = boot_fd(i, 3);
		}
	}
	for (i=0; (i<2) && (!bootseg); i++) {
		if (sxsi_hd[i].fname[0]) {
			bootseg = boot_hd(i | 0x80);
		}
	}
	for (i=0; (i<2) && (!bootseg); i++) {
		if (sxsi_hd[i+2].fname[0]) {
			bootseg = boot_hd(i | 0xa0);
		}
	}

	init_fdd_equip();
	init_sasi_equip();
	init_scsi_equip();
	return(bootseg);
}

// --------------------------------------------------------------------------

void bios0x1b(void) {

	BYTE	ret_ah;
	BYTE	flag;

	switch(I286_AL & 0xf0) {
		case 0x90:
			ret_ah = fdd_operate(DISKTYPE_2HD);
			break;

		case 0x10:
		case 0x70:
		case 0xf0:
			ret_ah = fdd_operate(DISKTYPE_2DD);
			break;

		case 0x50:
			ret_ah = fdd_operate(DISKTYPE_2D);
			break;

		case 0x00:
		case 0x80:
			ret_ah = sxsi_operate(HDDTYPE_SASI);
			break;
#if 0
		case 0x20:
		case 0xa0:
			ret_ah = sxsi_operate(HDDTYPE_SCSI);
			break;
#endif

		default:
			ret_ah = 0x40;
			break;
	}
#ifdef TRACE
	{
		char	buf[256];
		snprintf(buf, sizeof(buf),
		    "%04x:%04x AX=%04x BX=%04x %02x:%02x:%02x:%02x\n"
			"ES=%04x BP=%04x \nret=%02x",
			i286_memword_read(I286_SS, I286_SP+2),
			i286_memword_read(I286_SS, I286_SP),
			I286_AX, I286_BX, I286_CL, I286_DH, I286_DL, I286_CH, 
			I286_ES, I286_BP, ret_ah);
			TRACE_(buf, 0);
	}
#endif
	I286_AH = ret_ah;
	flag = i286_membyte_read(I286_SS, I286_SP+4) & 0xfe;
	if (ret_ah >= 0x20) {
		flag |= 1;
	}
	i286_membyte_write(I286_SS, I286_SP+4, flag);
}
