
%macro ADD_SEGBASE 1
%if 0
				cmp		byte [CPU_INST_SEGUSE], 0
				je		%%noused
				add		ecx, [CPU_INST_SEGBASE]
				jmp		short %%nexts
		%%noused:
				add		ecx, %1
		%%nexts:
%else
				cmp		byte [CPU_INST_SEGUSE], 0
				push	eax
				cmove	eax, %1
				cmovne	eax, [CPU_INST_SEGBASE]
				add		ecx, eax
				pop		eax
%endif
%endmacro

%macro GET_PREFIX_DS 0
%if 0
				cmp		byte [CPU_INST_SEGUSE], 0
				je		%%noused
				mov		edi, [CPU_INST_SEGBASE]
				ret
		%%noused:
				mov		edi, [DS_BASE]
				ret
%else
				cmp		byte [CPU_INST_SEGUSE], 0
				cmove	edi, [DS_BASE]
				cmovne	edi, [CPU_INST_SEGBASE]
				ret
%endif
%endmacro

%macro GET_PREFIX_SS 0
%if 0
				cmp		byte [CPU_INST_SEGUSE], 0
				je		%%noused
				mov		edi, [CPU_INST_SEGBASE]
				ret
		%%noused:
				mov		edi, [SS_BASE]
				ret
%else
				cmp		byte [CPU_INST_SEGUSE], 0
				cmove	edi, [SS_BASE]
				cmovne	edi, [CPU_INST_SEGBASE]
				ret
%endif
%endmacro

; ----------------------------------------

					; dest: CPU_REG[eax] src: dl
%macro PREPART_EA_REG8 1
				GET_PCBYTE
				movzx	ebx, al
				shr		ebx, 2
				bt		ebx, 3
				rcl		ebx, 1
				and		ebx, 0dh
				cmp		al, 0c0h
				jc		.memory_eareg8
				shl		eax, 1
				bt		eax, 3
				rcl		eax, 1
				and		eax, 0dh
				mov		dl, [CPU_REG + ebx]
				CPUCLOCK %1
%endmacro

					; dest: MAIN_MEM[ecx] src: dl
%macro MEMORY_EA_REG8 1
			.memory_eareg8:
				CPUCLOCK %1
				rol		al, 2
				and		eax, 1fh
				call	calc_ea_dst
				cmp		ecx, MAINMEM_WRITEMAX
				jae		.extmem_eareg8
				mov		dl, [CPU_REG + ebx]
%endmacro

					; dest: al,ecx src: CPU_REG[ebx]
%macro EXTMEM_EA_REG8 0
			.extmem_eareg8:
				call	__f_i286_memoryread@4
%endmacro

; ----------------------------------------

					; dest: CPU_REG[eax*4] src: dx
%macro PREPART_EA_REG16 1
				GET_PCBYTE
				movzx	ebx, al
				shr		ebx, 3
				and		ebx, 7
				cmp		al, 0c0h
				jc		.memory_eareg16
				and		eax, 7
				mov		dx, [CPU_REG + ebx * 4]
				CPUCLOCK %1
%endmacro

					; dest: MAIN_MEM[ecx] src: dx
%macro MEMORY_EA_REG16 1
			.memory_eareg16:
				CPUCLOCK %1
				rol		al, 2
				and		eax, 1fh
				call	calc_ea_dst
				cmp		ecx, MAINMEM_WRITEMAX - 1
				jae		.extmem_eareg16
				mov		dx, [CPU_REG + ebx * 4]
%endmacro

					; dest: ax,ecx src: CPU_REG[ebx*4]
%macro EXTMEM_EA_REG16 0
			.extmem_eareg16:
				call	__f_i286_memoryread_w@4
%endmacro

; ----------------------------------------

					; dest: CPU_REG[eax*4] src: dx
%macro PREPART_EA_REG32 1
				GET_PCBYTE
				movzx	ebx, al
				shr		ebx, 3
				and		ebx, 7
				cmp		al, 0c0h
				jc		.memory_eareg32
				and		eax, 7
				mov		edx, [CPU_REG + ebx * 4]
				CPUCLOCK %1
%endmacro

					; dest: MAIN_MEM[ecx] src: dx
%macro MEMORY_EA_REG32 1
			.memory_eareg32:
				CPUCLOCK %1
				rol		al, 2
				and		eax, 1fh
				call	calc_ea_dst
				cmp		ecx, MAINMEM_WRITEMAX - 3
				jae		.extmem_eareg32
				mov		edx, [CPU_REG + ebx * 4]
%endmacro

					; dest: ax,ecx src: CPU_REG[ebx*4]
%macro EXTMEM_EA_REG32 0
			.extmem_eareg32:
				call	__f_i286_memoryread_d@4
%endmacro

; ----------------------------------------

					; dest: CPU_REG[ebx]  src: al
%macro PREPART_REG8_EA 2
				GET_PCBYTE
				movzx	ebx, al
				shr		ebx, 2
				bt		ebx, 3
				rcl		ebx, 1
				and		ebx, 0dh
				cmp		al, 0c0h
				jnc		.src_register
				CPUCLOCK %2
				rol		al, 2
				and		eax, 1fh
				call	calc_ea_dst
				call	__f_i286_memoryread@4
				jmp		short .reg8_ea_ready
			.src_register:
				shl		eax, 1
				bt		eax, 3
				rcl		eax, 1
				and		eax, 0dh
				mov		al, [CPU_REG + eax]
				CPUCLOCK %1
			.reg8_ea_ready:
%endmacro

; ----------------------------------------

					; dest: CPU_REG[ebx*4]  src: ax
%macro PREPART_REG16_EA 2
				GET_PCBYTE
				movzx	ebx, al
				shr		ebx, 3
				and		ebx, 7
				cmp		al, 0c0h
				jnc		.src_register
				CPUCLOCK %2
				rol		al, 2
				and		eax, 1fh
				call	calc_ea_dst
				call	__f_i286_memoryread_w@4
				jmp		short .reg16_ea_ready
			.src_register:
				and		eax, 7
				mov		ax, [CPU_REG + eax * 4]
				CPUCLOCK %1
			.reg16_ea_ready:
%endmacro

; ----------------------------------------

					; dest: CPU_REG[ebx*4]  src: ax
%macro PREPART_REG32_EA 2
				GET_PCBYTE
				movzx	ebx, al
				shr		ebx, 3
				and		ebx, 7
				cmp		al, 0c0h
				jnc		.src_register
				CPUCLOCK %2
				rol		al, 2
				and		eax, 1fh
				call	calc_ea_dst
				call	__f_i286_memoryread_d@4
				jmp		short .reg32_ea_ready
			.src_register:
				and		eax, 7
				mov		eax, [CPU_REG + eax * 4]
				CPUCLOCK %1
			.reg32_ea_ready:
%endmacro

; ----------------------------------------

					; dest: CPU_REG[eax]
%macro PREPART_EA8 1
				cmp		al, 0c0h
				jc		.memory_eareg8
				CPUCLOCK %1
				shl		eax, 1
				bt		eax, 3
				rcl		eax, 1
				and		eax, 0dh
%endmacro

					; dest: MAIN_MEM[ecx]
%macro MEMORY_EA8 1
			.memory_eareg8:
				CPUCLOCK %1
				rol		al, 2
				and		eax, 1fh
				call	calc_ea_dst
				cmp		ecx, MAINMEM_WRITEMAX
				jae		.extmem_eareg8
%endmacro

					; dest: ecx
%macro EXTMEM_EA8 0
			.extmem_eareg8:
				call	__f_i286_memoryread@4
%endmacro

; ----------------------------------------
					; dest: CPU_REG[eax*4]
%macro PREPART_EA16 1
				cmp		al, 0c0h
				jc		.memory_eareg16
				CPUCLOCK %1
				and		eax, 7
%endmacro

					; dest: MAIN_MEM[ecx]
%macro MEMORY_EA16 1
			.memory_eareg16:
				CPUCLOCK %1
				rol		al, 2
				and		eax, 1fh
				call	calc_ea_dst
				cmp		ecx, MAINMEM_WRITEMAX - 1
				jae		.extmem_eareg16
%endmacro

					; dest: ecx
%macro EXTMEM_EA16 0
			.extmem_eareg16:
				call	__f_i286_memoryread_w@4
%endmacro

; ----------------------------------------
					; dest: CPU_REG[eax*4]
%macro PREPART_EA32 1
				cmp		al, 0c0h
				jc		.memory_eareg32
				CPUCLOCK %1
				and		eax, 7
%endmacro

					; dest: MAIN_MEM[ecx]
%macro MEMORY_EA32 1
			.memory_eareg32:
				CPUCLOCK %1
				rol		al, 2
				and		eax, 1fh
				call	calc_ea_dst
				cmp		ecx, MAINMEM_WRITEMAX - 3
				jae		.extmem_eareg32
%endmacro

					; dest: ecx
%macro EXTMEM_EA32 0
			.extmem_eareg32:
				call	__f_i286_memoryread_d@4
%endmacro

