
/*
	Intel Architecture 32-bit Processor Interpreter Engine for Pentium

				Copyright by Yui/Studio Milmake 1999-2000
				Copyright by Norio HATTORI 2000,2001
				Copyright by NONAKA Kimihiro 2002
*/

#ifndef NP2_CPU_H__
#define NP2_CPU_H__

#include "compiler.h"

#ifdef __cplusplus
extern "C" {
#endif

typedef union {
#if defined(NP2_BIGENDIAN)
	struct {
		BYTE	_hh;
		BYTE	_hl;
		BYTE	h;
		BYTE	l;
	} b;
	struct {
		WORD	_hw;
		WORD	w;
	}
#else	/* NP2_LITTLEENDIAN */
	struct {
		BYTE	l;
		BYTE	h;
		BYTE	_hl;
		BYTE	_hh;
	} b;
	struct {
		WORD	w;
		WORD	_hw;
	} w;
#endif
	DWORD	d;
} __attribute__((packed)) REG32;

typedef struct {
	union {
		struct {
			REG32	eax;
			REG32	ecx;
			REG32	edx;
			REG32	ebx;
			REG32	esp;
			REG32	ebp;
			REG32	esi;
			REG32	edi;
		} r;
		REG32	reg[8];
	} u;

	WORD	es;
	WORD	cs;
	WORD	ss;
	WORD	ds;
	WORD	fs;
	WORD	gs;

	REG32	eflags;

	REG32	eip;
} __attribute__((packed)) CPU_REGS;

typedef struct {
	WORD	gdtr_limit;
	DWORD	gdtr_base;
	WORD	idtr_limit;
	DWORD	idtr_base;
	WORD	ldtr;
	WORD	tr;

	DWORD	cr0;
	DWORD	cr1;
	DWORD	cr2;
	DWORD	cr3;
	DWORD	cr4;
	DWORD	mxcsr;
} __attribute__((packed)) CPU_SYSREGS;

typedef struct {
	DWORD	es_base;
	DWORD	cs_base;
	DWORD	ss_base;
	DWORD	ds_base;
	DWORD	fs_base;
	DWORD	gs_base;

	BYTE	trap;
	BYTE	padding;
	DWORD	inport;
	BYTE	cpu_type;
	BYTE	_dummy;
	DWORD	ovflag;
} __attribute__((packed)) CPU_STAT;

typedef struct {
	BYTE	op_32;
	BYTE	as_32;
	BYTE	rep_used;
	BYTE	seg_used;
	DWORD	seg_base;
} __attribute__((packed)) CPU_INST;

extern CPU_REGS		cpu_regs;
extern CPU_SYSREGS	cpu_sysregs;
extern CPU_STAT		cpu_stat;
extern CPU_INST		cpu_inst;
extern BYTE 		iflags[];

#define	CPU_FAMILY	6
#define	CPU_MODEL	1
#define	CPU_STEPPING	3

#define	CPU_REGS_BYTEL(n)	cpu_regs.u.reg[(n)].b.l
#define	CPU_REGS_BYTEH(n)	cpu_regs.u.reg[(n)].b.h
#define	CPU_REGS_WORD(n)	cpu_regs.u.reg[(n)].w.w
#define	CPU_REGS_DWORD(n)	cpu_regs.u.reg[(n)].d

#define I286_AL		cpu_regs.u.r.eax.b.l
#define I286_BL		cpu_regs.u.r.ebx.b.l
#define I286_CL		cpu_regs.u.r.ecx.b.l
#define I286_DL		cpu_regs.u.r.edx.b.l
#define I286_AH		cpu_regs.u.r.eax.b.h
#define I286_BH		cpu_regs.u.r.ebx.b.h
#define I286_CH		cpu_regs.u.r.ecx.b.h
#define I286_DH		cpu_regs.u.r.edx.b.h
#define CPU_AL		cpu_regs.u.r.eax.b.l
#define CPU_BL		cpu_regs.u.r.ebx.b.l
#define CPU_CL		cpu_regs.u.r.ecx.b.l
#define CPU_DL		cpu_regs.u.r.edx.b.l
#define CPU_AH		cpu_regs.u.r.eax.b.h
#define CPU_BH		cpu_regs.u.r.ebx.b.h
#define CPU_CH		cpu_regs.u.r.ecx.b.h
#define CPU_DH		cpu_regs.u.r.edx.b.h

#define I286_AX		cpu_regs.u.r.eax.w.w
#define I286_BX		cpu_regs.u.r.ebx.w.w
#define I286_CX		cpu_regs.u.r.ecx.w.w
#define I286_DX		cpu_regs.u.r.edx.w.w
#define I286_SI		cpu_regs.u.r.esi.w.w
#define I286_DI		cpu_regs.u.r.edi.w.w
#define I286_BP		cpu_regs.u.r.ebp.w.w
#define I286_SP		cpu_regs.u.r.esp.w.w
#define I286_IP		cpu_regs.eip.w.w
#define CPU_AX		cpu_regs.u.r.eax.w.w
#define CPU_BX		cpu_regs.u.r.ebx.w.w
#define CPU_CX		cpu_regs.u.r.ecx.w.w
#define CPU_DX		cpu_regs.u.r.edx.w.w
#define CPU_SI		cpu_regs.u.r.esi.w.w
#define CPU_DI		cpu_regs.u.r.edi.w.w
#define CPU_BP		cpu_regs.u.r.ebp.w.w
#define CPU_SP		cpu_regs.u.r.esp.w.w
#define CPU_IP		cpu_regs.eip.w.w

#define I286_EAX	cpu_regs.u.r.eax.d
#define I286_EBX	cpu_regs.u.r.ebx.d
#define I286_ECX	cpu_regs.u.r.ecx.d
#define I286_EDX	cpu_regs.u.r.edx.d
#define I286_ESI	cpu_regs.u.r.esi.d
#define I286_EDI	cpu_regs.u.r.edi.d
#define I286_EBP	cpu_regs.u.r.ebp.d
#define I286_ESP	cpu_regs.u.r.esp.d
#define I286_EIP	cpu_regs.eip.d
#define CPU_EAX		cpu_regs.u.r.eax.d
#define CPU_EBX		cpu_regs.u.r.ebx.d
#define CPU_ECX		cpu_regs.u.r.ecx.d
#define CPU_EDX		cpu_regs.u.r.edx.d
#define CPU_ESI		cpu_regs.u.r.esi.d
#define CPU_EDI		cpu_regs.u.r.edi.d
#define CPU_EBP		cpu_regs.u.r.ebp.d
#define CPU_ESP		cpu_regs.u.r.esp.d
#define CPU_EIP		cpu_regs.eip.d

#define I286_CS		cpu_regs.cs
#define I286_DS		cpu_regs.ds
#define I286_ES		cpu_regs.es
#define I286_FS		cpu_regs.fs
#define I286_GS		cpu_regs.gs
#define I286_SS		cpu_regs.ss
#define CPU_CS		cpu_regs.cs
#define CPU_DS		cpu_regs.ds
#define CPU_ES		cpu_regs.es
#define CPU_FS		cpu_regs.fs
#define CPU_GS		cpu_regs.gs
#define CPU_SS		cpu_regs.ss

#define SEG_BASE	cpu_stat.es_base
#define ES_BASE		cpu_stat.es_base
#define CS_BASE		cpu_stat.cs_base
#define SS_BASE		cpu_stat.ss_base
#define DS_BASE		cpu_stat.ds_base
#define FS_BASE		cpu_stat.fs_base
#define GS_BASE		cpu_stat.gs_base

#define I286_EFLAG	cpu_regs.eflags.d
#define I286_FLAG	cpu_regs.eflags.w.w
#define I286_FLAGL	cpu_regs.eflags.b.l
#define I286_FLAGH	cpu_regs.eflags.b.h
#define I286_TRAP	cpu_stat.trap
#define I286_INPORT	cpu_stat.inport
#define I286_OV		cpu_stat.ovflag
#define i286s		cpu_stat
#define CPU_EFLAG	cpu_regs.eflags.d
#define CPU_FLAG	cpu_regs.eflags.w.w
#define CPU_FLAGL	cpu_regs.eflags.b.l
#define CPU_FLAGH	cpu_regs.eflags.b.h
#define CPU_TRAP	cpu_stat.trap
#define CPU_INPORT	cpu_stat.inport
#define CPU_OV		cpu_stat.ovflag

#define C_FLAG		(1 << 0)
#define P_FLAG		(1 << 2)
#define A_FLAG		(1 << 4)
#define Z_FLAG		(1 << 6)
#define S_FLAG		(1 << 7)
#define T_FLAG		(1 << 8)
#define I_FLAG		(1 << 9)
#define D_FLAG		(1 << 10)
#define O_FLAG		(1 << 11)

#define IOPL_FLAG	(3 << 12)
#define NT_FLAG		(1 << 14)
#define RF_FLAG		(1 << 16)
#define VM_FLAG		(1 << 17)
#define AC_FLAG		(1 << 18)
#define VIF_FLAG	(1 << 19)
#define VIP_FLAG	(1 << 20)
#define ID_FLAG		(1 << 21)

#define	REAL_FLAGREG	((CPU_FLAG & 0x07ff) | (CPU_OV ? O_FLAG : 0))
#define	REAL_EFLAGREG	((CPU_EFLAG & 0xfffff7ff) | (CPU_OV ? O_FLAG : 0))

#define CPUTYPE		cpu_stat.cpu_type
#define CPUTYPE_V30	0x01

#define	CPU_INST_OP32		cpu_inst.op_32
#define	CPU_INST_AS32		cpu_inst.as_32
#define	CPU_INST_REPUSE		cpu_inst.rep_used
#define	CPU_INST_SEGUSE		cpu_inst.seg_used
#define	CPU_INST_SEGBASE	cpu_inst.seg_base

#define isI286DI	(!(I286_FLAG & I_FLAG))
#define isI286EI	(I286_FLAG & I_FLAG)
#define I286_CLI	do { I286_FLAG &= ~I_FLAG;	\
					I286_TRAP = 0; } while (/*CONSTCOND*/ 0)
#define I286_STI	do { I286_FLAG |= I_FLAG;	\
					I286_TRAP = (I286_FLAG >> 8) & 1; } while (/*CONSTCOND*/ 0)
#define CPU_CLI		do { CPU_FLAG &= ~I_FLAG;	\
					CPU_TRAP = 0; } while (/*CONSTCOND*/ 0)
#define CPU_STI		do { CPU_FLAG |= I_FLAG;	\
					CPU_TRAP = (CPU_FLAG >> 8) & 1; } while (/*CONSTCOND*/0)

#define CPU_GDTR_LIMIT	cpu_sysregs.gdtr_limit
#define CPU_GDTR_BASE	cpu_sysregs.gdtr_base
#define CPU_IDTR_LIMIT	cpu_sysregs.idtr_limit
#define CPU_IDTR_BASE	cpu_sysregs.idtr_base
#define CPU_LDTR	cpu_sysregs.ldtr
#define CPU_TR		cpu_sysregs.tr

#define CPU_CR0		cpu_sysregs.cr0
#define CPU_CR1		cpu_sysregs.cr1
#define CPU_CR2		cpu_sysregs.cr2
#define CPU_CR3		cpu_sysregs.cr3
#define CPU_CR4		cpu_sysregs.cr4
#define CPU_MXCSR	cpu_sysregs.mxcsr

#define	CR0_PE		(1 << 0)
#define	CR0_MP		(1 << 1)
#define	CR0_EM		(1 << 2)
#define	CR0_TS		(1 << 3)
#define	CR0_ET		(1 << 4)
#define	CR0_NE		(1 << 5)
#define	CR0_WP		(1 << 16)
#define	CR0_AM		(1 << 18)
#define	CR0_NW		(1 << 29)
#define	CR0_CD		(1 << 30)
#define	CR0_PG		(1 << 31)

void ia32reset(void);
void ia32(void);
void ia32withtrap(void);
void ia32withdma(void);

void ia32_step(void);
void INTR_CALL ia32_interrupt(BYTE vect);

#define	CPU_EXEC()		ia32()
#define	CPU_WITHTRAP()		ia32withtrap()
#define	CPU_WITHDMA()		ia32withdma()
#define	CPU_STEP()		ia32_step()
#define	CPU_INTERRUPT(a)	ia32_interrupt(a)
#define	cpu_reset()		ia32reset()

void exec_1step(void);
#define	INST_PREFIX	(1 << 0)
#define	INST_STRING	(1 << 1)
#define	REP_CHECKZF	(1 << 7)

extern BYTE  *reg8_b20[0x100];
extern BYTE  *reg8_b53[0x100];
extern WORD  *reg16_b20[0x100];
extern WORD  *reg16_b53[0x100];
extern DWORD *reg32_b20[0x100];
extern DWORD *reg32_b53[0x100];

#define	MAINMEM_READMAX		0xa4000
#define	MAINMEM_WRITEMAX	0xa0000

extern DWORD EA_FIX;


#ifdef __cplusplus
}
#endif

#endif	/* !NP2_CPU_H__ */
