#include	"compiler.h"

#include	"iobridge.h"
#include	"dmac.h"
#include	"fdc.h"
#include	"fddfile.h"
#include	"fdd_xdf.h"
#include	"fdd_d88.h"
#include	"milstr.h"
#include	"dosio.h"

	FDCFILE_TABLE	fddfile[4];
	BYTE			fddlasterror;

// --------------------------------------------------------------------------

void fddfile_init(void) {

	ZeroMemory(fddfile, sizeof(fddfile));
}

void fddfile_reset2dmode(void) {

	int		i;

	for (i=0; i<4; i++) {
		fddfile[i].mode2d = 0;
	}
}

char *fdd_diskname(BYTE drv) {

	if (drv >= 4) {
		return("");
	}
	return(fddfile[drv].fname);
}

BOOL fdd_diskready(BYTE drv) {

	if ((drv >= 4) || (!fddfile[drv].fname[0])) {
		return(FALSE);
	}
	return(TRUE);
}

BOOL fdd_diskprotect(BYTE drv) {

	if ((drv >= 4) || (!fddfile[drv].protect)) {
		return(FALSE);
	}
	return(TRUE);
}

// --------------------------------------------------------------------------

BOOL fdd_set(BYTE drv, LPSTR fname) {

	char	*p;
	int		ftype;

	if (drv >= 4) {
		return(FAILURE);
	}
	ftype = file_getftype(fname);
	if ((ftype != FTYPE_D88) && (ftype != FTYPE_BETA)) {
		p = getExtName(fname);
		if ((!milstr_cmpA(p, "D88")) || (!milstr_cmpA(p, "88D")) ||
			(!milstr_cmpA(p, "D98")) || (!milstr_cmpA(p, "98D"))) {
			ftype = FTYPE_D88;
		}
	}
	switch(ftype) {
		case FTYPE_D88:
			return(fdd_set_d88(drv, fname));
	}
	return(fdd_set_xdf(drv, fname));
}

BOOL fdd_eject(BYTE drv) {

	if (drv >= 4) {
		return(FAILURE);
	}
	switch(fddfile[drv].type) {
		case DISKTYPE_BETA:
			return(fdd_eject_xdf(drv));
		case DISKTYPE_D88:
			return(fdd_eject_d88(drv));
	}
	return(FAILURE);
}

// --------------------------------------------------------------------------

BOOL fdd_diskaccess(void) {

	switch(fddfile[fdc.us].type) {
		case DISKTYPE_BETA:
			return(fdd_diskaccess_xdf());
		case DISKTYPE_D88:
			return(fdd_diskaccess_d88());
	}
	return(FAILURE);
}

BOOL fdd_seek(void) {

	BOOL	ret = FAILURE;

	switch(fddfile[fdc.us].type) {
		case DISKTYPE_BETA:
			ret = fdd_seek_xdf();
			break;
		case DISKTYPE_D88:
			ret = fdd_seek_d88();
			break;
	}
	fdc.treg[fdc.us] = fdc.ncn;
	return(ret);
}

BOOL fdd_seeksector(void) {

	switch(fddfile[fdc.us].type) {
		case DISKTYPE_BETA:
			return(fdd_seeksector_xdf());
		case DISKTYPE_D88:
			return(fdd_seeksector_d88());
	}
	return(FAILURE);
}


BOOL fdd_read(void) {

	switch(fddfile[fdc.us].type) {
		case DISKTYPE_BETA:
			return(fdd_read_xdf());
		case DISKTYPE_D88:
			return(fdd_read_d88());
	}
	return(FAILURE);
}

BOOL fdd_write(void) {

	switch(fddfile[fdc.us].type) {
		case DISKTYPE_BETA:
			return(fdd_write_xdf());
		case DISKTYPE_D88:
			return(fdd_write_d88());
	}
	return(FAILURE);
}

BOOL fdd_readid(void) {

	switch(fddfile[fdc.us].type) {
		case DISKTYPE_BETA:
			return(fdd_readid_xdf());
		case DISKTYPE_D88:
			return(fdd_readid_d88());
	}
	return(FAILURE);
}

BOOL fdd_formatinit(void) {

	if (fddfile[fdc.us].type == DISKTYPE_D88) {
		return(fdd_formatinit_d88());
	}
	return(FAILURE);
}

BOOL fdd_formating(BYTE *ID) {

	if (fddfile[fdc.us].type == DISKTYPE_D88) {
		return(fdd_formating_d88(ID));
	}
	return(FAILURE);
}

BOOL fdd_isformating(void) {

	if (fddfile[fdc.us].type == DISKTYPE_D88) {
		return(fdd_isformating_d88());
	}
	return(FALSE);
}

