/*	$Id: gtk_about.c,v 1.3 2002/06/03 15:46:52 nonaka Exp $	*/

/*
 * Copyright (c) 2002 NONAKA Kimihiro <aw9k-nnk@asahi-net.or.jp>
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer as
 *    the first lines of this file unmodified.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include "compiler.h"

#include "np2.h"
#include "pc_core.h"
#include "ini.h"
#include "xnp2.h"
#include "xmenu.h"
#include "version.h"

#include "icons/xnp2.xpm"

static void
about_destroy(GtkWidget *w, GtkWidget **wp)
{

	UNUSED(wp);

	install_idle_process();
	gtk_widget_destroy(w);
}

void
create_about_dialog(void)
{
	GtkWidget *about_window;
	GtkWidget *hbox1;
	GtkWidget *neko_pixmap;
	GtkWidget *version_labal;
	GtkWidget *vbox1;
	GtkWidget *ok_button;
	GdkColormap *colormap;
	GdkPixmap *gdkpixmap;
	GdkBitmap *mask;

	uninstall_idle_process();

	about_window = gtk_window_new(GTK_WINDOW_DIALOG);
	gtk_object_set_data(GTK_OBJECT(about_window),"About Xnp2",about_window);
	gtk_widget_set_usize(about_window, 320, 80);
	gtk_window_set_title(GTK_WINDOW(about_window), "About Xnp2");
	gtk_window_set_position(GTK_WINDOW(about_window), GTK_WIN_POS_CENTER);
	gtk_window_set_modal(GTK_WINDOW(about_window), TRUE);
	gtk_window_set_policy(GTK_WINDOW(about_window), FALSE, FALSE, FALSE);

	gtk_signal_connect(GTK_OBJECT(about_window), "destroy",
	    GTK_SIGNAL_FUNC(about_destroy), NULL);

	hbox1 = gtk_hbox_new(FALSE, 0);
	gtk_widget_ref(hbox1);
	gtk_object_set_data_full(GTK_OBJECT (about_window), "hbox1", hbox1,
	    (GtkDestroyNotify)gtk_widget_unref);
	gtk_widget_show(hbox1);
	gtk_container_add(GTK_CONTAINER(about_window), hbox1);

	colormap = gtk_widget_get_colormap(about_window);
	gdkpixmap = gdk_pixmap_colormap_create_from_xpm_d(NULL, colormap,
	    &mask, NULL, np2_icon);
	if (gdkpixmap == NULL)
		g_error("Couldn't create replacement pixmap.");
	neko_pixmap = gtk_pixmap_new(gdkpixmap, mask);
	gdk_pixmap_unref(gdkpixmap);
	gdk_bitmap_unref(mask);

	gtk_widget_ref(neko_pixmap);
	gtk_object_set_data_full(GTK_OBJECT(about_window), "neko_pixmap",
	    neko_pixmap, (GtkDestroyNotify)gtk_widget_unref);
	gtk_widget_show(neko_pixmap);
	gtk_box_pack_start(GTK_BOX(hbox1), neko_pixmap, FALSE, FALSE, 0);
	gtk_misc_set_padding(GTK_MISC(neko_pixmap), 12, 0);

	version_labal = gtk_label_new("Neko Project II for X\n(Based on Pre Release 4)\nSource Version " SOURCE_VERSION);
	gtk_widget_ref(version_labal);
	gtk_object_set_data_full(GTK_OBJECT(about_window), "version_labal",
	    version_labal, (GtkDestroyNotify)gtk_widget_unref);
	gtk_widget_show(version_labal);
	gtk_box_pack_start(GTK_BOX(hbox1), version_labal, FALSE, FALSE, 0);
	gtk_label_set_justify(GTK_LABEL(version_labal), GTK_JUSTIFY_LEFT);
	gtk_label_set_line_wrap(GTK_LABEL(version_labal), TRUE);
	gtk_misc_set_padding(GTK_MISC(version_labal), 10, 0);

	vbox1 = gtk_vbox_new(FALSE, 0);
	gtk_widget_ref(vbox1);
	gtk_object_set_data_full(GTK_OBJECT(about_window), "vbox1", vbox1,
	    (GtkDestroyNotify)gtk_widget_unref);
	gtk_widget_show(vbox1);
	gtk_box_pack_start(GTK_BOX(hbox1), vbox1, FALSE, FALSE, 0);

	ok_button = gtk_button_new_with_label("OK");
	gtk_widget_ref(ok_button);
	gtk_object_set_data_full(GTK_OBJECT(about_window), "ok_button",
	    ok_button, (GtkDestroyNotify)gtk_widget_unref);
	gtk_widget_show(ok_button);
	gtk_widget_set_usize(ok_button, 80, 32);
	gtk_box_pack_start(GTK_BOX(vbox1), ok_button, FALSE, FALSE, 0);
	gtk_signal_connect_object(GTK_OBJECT(ok_button), "clicked",
	    GTK_SIGNAL_FUNC(gtk_widget_destroy), GTK_OBJECT(about_window));
	GTK_WIDGET_SET_FLAGS(ok_button, GTK_CAN_DEFAULT);
	GTK_WIDGET_SET_FLAGS(ok_button, GTK_HAS_DEFAULT);
	gtk_widget_grab_default(ok_button);

	gtk_widget_show_all(about_window);
}
