/*	$Id: gtk_file_selection.c,v 1.10 2002/09/10 16:47:02 nonaka Exp $	*/

/*
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgment:
 *      This product includes software developed by NONAKA Kimihiro.
 * 4. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include "compiler.h"

#include <sys/types.h>
#include <sys/stat.h>

#include "xnp2.h"
#include "xmenu.h"

#include "iobridge.h"
#include "diskdrv.h"
#include "font.h"
#include "gdc.h"
#include "ini.h"

static int file_type;
static int drive;

static void
file_selection_ok(GtkWidget *w, GtkFileSelection *fs)
{
	struct stat st;
	char *p;

	UNUSED(w);

	p = gtk_file_selection_get_filename(fs);
	if (p != NULL && stat(p, &st) == 0) {
		if (!S_ISDIR(st.st_mode)) {
			switch (file_type) {
			case FILE_TYPE_FDD:
				diskdrv_insertfdd(drive, p);
				strncpy(fddfolder, p, MAX_PATH);
				INIT_RENEWAL;
				break;

			case FILE_TYPE_SASI:
				diskdrv_changehdd(drive, p);
				strncpy(hddfolder, p, MAX_PATH);
				INIT_RENEWAL;
				break;

			case FILE_TYPE_FONT:
				if (font_load(p, FALSE) != 0) {
					INIT_RENEWAL;
					gdcs.textdisp |= GDCSCRN_ALLDRAW2;
					strncpy(fontfile, p, MAX_PATH);
				}
				break;
			}
		}
	}
	gtk_widget_destroy(GTK_WIDGET(fs));
}

static void
file_selection_destroy(GtkWidget *w, GtkWidget **wp)
{

	UNUSED(wp);

	install_idle_process();
	gtk_widget_destroy(w);
}

void
create_file_selection(int type, char *title, char *defstr, int arg)
{
	GtkWidget *file_dialog;

	uninstall_idle_process();

	file_type = type;
	drive = arg;

	file_dialog = gtk_file_selection_new(title);
	if (defstr != NULL) {
		gtk_file_selection_set_filename(
		    GTK_FILE_SELECTION(file_dialog), defstr);
	}
	gtk_window_set_position(GTK_WINDOW(file_dialog), GTK_WIN_POS_CENTER);
	gtk_window_set_modal(GTK_WINDOW(file_dialog), TRUE);
	gtk_file_selection_hide_fileop_buttons(GTK_FILE_SELECTION(file_dialog));
	gtk_signal_connect(GTK_OBJECT(file_dialog), "destroy",
	    GTK_SIGNAL_FUNC(file_selection_destroy), NULL);
	gtk_signal_connect(
	    GTK_OBJECT(GTK_FILE_SELECTION(file_dialog)->ok_button),
	    "clicked", GTK_SIGNAL_FUNC(file_selection_ok), file_dialog);
	gtk_signal_connect_object(
	    GTK_OBJECT(GTK_FILE_SELECTION(file_dialog)->cancel_button),
	    "clicked", GTK_SIGNAL_FUNC(gtk_widget_destroy),
	    GTK_OBJECT(file_dialog));

	gtk_widget_show(file_dialog);
}
