/*	$Id: gtk_screen.c,v 1.6 2002/07/09 16:05:42 nonaka Exp $	*/

/*
 * Copyright (c) 2002 NONAKA Kimihiro <aw9k-nnk@asahi-net.or.jp>
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer as
 *    the first lines of this file unmodified.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include "compiler.h"

#include "np2.h"
#include "pc_core.h"
#include "ini.h"
#include "np2pc98.h"
#include "xnp2.h"
#include "xmenu.h"

static GtkWidget *lcd_checkbutton;
static GtkWidget *reverse_checkbutton;
static GtkWidget *skipline_checkbutton;
static GtkWidget *color16_checkbutton;
static GtkObject *ratio_adj;
static GtkObject *tram_adj;
static GtkObject *vram_adj;
static GtkObject *grcg_adj;

static gint uPD72020;
static gint gc_kind;

static void
ok_button_clicked(GtkButton *b, gpointer d)
{
	gint lcd = GTK_TOGGLE_BUTTON(lcd_checkbutton)->active;
	gint lcdrev = GTK_TOGGLE_BUTTON(reverse_checkbutton)->active;
	gint skipline = GTK_TOGGLE_BUTTON(skipline_checkbutton)->active;
	gint color16 = GTK_TOGGLE_BUTTON(color16_checkbutton)->active;
	guint ratio = (guint)GTK_ADJUSTMENT(ratio_adj)->value;
	guint tram = (guint)GTK_ADJUSTMENT(tram_adj)->value;
	guint vram = (guint)GTK_ADJUSTMENT(vram_adj)->value;
	guint grcg = (guint)GTK_ADJUSTMENT(grcg_adj)->value;

	UNUSED(b);

	/* Video tab */
	if ((np2cfg.LCD_MODE & 1) != lcd) {
		INIT_RENEWAL;
		if (lcd) {
			np2cfg.LCD_MODE |= 1;
		} else {
			np2cfg.LCD_MODE &= ~1;
		}
	}
	if (((np2cfg.LCD_MODE & 2) ? 1 : 0) != lcdrev) {
		INIT_RENEWAL;
		if (lcdrev) {
			np2cfg.LCD_MODE |= 2;
		} else {
			np2cfg.LCD_MODE &= ~2;
		}
		np2pal_initpalettes();
	}
	if (np2cfg.skipline != skipline) {
		INIT_RENEWAL;
		np2cfg.skipline = skipline;
	}
	if (np2cfg.skiplight != ratio) {
		INIT_RENEWAL;
		np2cfg.skiplight = ratio;
	}

	/* Chip tab */
	if (np2cfg.uPD72020 != uPD72020) {
		INIT_RENEWAL;
		np2cfg.uPD72020 = uPD72020;
	}
	if ((np2cfg.grcg & 3) != gc_kind) {
		INIT_RENEWAL;
		np2cfg.grcg = (np2cfg.grcg & ~3) | gc_kind;
	}
	if (np2cfg.color16 != color16) {
		INIT_RENEWAL;
		np2cfg.color16 = color16;
	}

	/* Timing tab */
	if (np2cfg.wait[0] != tram) {
		INIT_RENEWAL;
		np2cfg.wait[0] = tram;
	}
	if (np2cfg.wait[2] != vram) {
		INIT_RENEWAL;
		np2cfg.wait[2] = vram;
	}
	if (np2cfg.wait[4] != grcg) {
		INIT_RENEWAL;
		np2cfg.wait[4] = grcg;
	}

	gtk_widget_destroy((GtkWidget *)d);
}

static void
screen_destroy(GtkWidget *w, GtkWidget **wp)
{

	UNUSED(wp);

	install_idle_process();
	gtk_widget_destroy(w);
}

static void
lcd_checkbutton_clicked(GtkCheckButton *b, gpointer d)
{

	gtk_widget_set_sensitive((GtkWidget *)d, GTK_TOGGLE_BUTTON(b)->active);
}

static void
uPD72020_radiobutton_clicked(GtkButton *b, gpointer d)
{

	UNUSED(b);

	uPD72020 = (gint)d;
}

static void
gc_radiobutton_clicked(GtkButton *b, gpointer d)
{

	UNUSED(b);

	gc_kind = (gint)d;
}

void
scale_set_default_values(GtkScale *scale)
{

	gtk_range_set_update_policy(GTK_RANGE(scale), GTK_UPDATE_CONTINUOUS);
	gtk_scale_set_digits(scale, 1);
	gtk_scale_set_value_pos(scale, GTK_POS_RIGHT);
	gtk_scale_set_draw_value(scale, TRUE);
}

void
create_screen_dialog(void)
{
	GtkWidget *screen_window;
	GtkWidget *ratio_scale;
	GtkWidget *tram_scale;
	GtkWidget *vram_scale;
	GtkWidget *grcg_scale;
	GtkWidget *upd72020_radiobutton[2];
	GtkWidget *gc_radiobutton[4];
	GtkWidget *vbox2;
	GtkWidget *screen_notebook;
	GtkWidget *vbox3;
	GtkWidget *hbox5;
	GtkWidget *ratio_label;
	GtkWidget *video_label;
	GtkWidget *vbox4;
	GtkWidget *gdc_frame;
	GtkWidget *hbox6;
	GSList *hbox6_group = NULL;
	GtkWidget *gc_frame;
	GtkWidget *hbox7;
	GSList *hbox7_group = NULL;
	GtkWidget *chip_label;
	GtkWidget *vbox5;
	GtkWidget *hbox8;
	GtkWidget *tram_label;
	GtkWidget *tramclock_label;
	GtkWidget *hbox9;
	GtkWidget *vram_label;
	GtkWidget *vramclock_label;
	GtkWidget *hbox10;
	GtkWidget *grcg_label;
	GtkWidget *grcgclock_label;
	GtkWidget *timing_label;
	GtkWidget *hbox1;
	GtkWidget *null_label;
	GtkWidget *ok_button;
	GtkWidget *cancel_button;
	int i;

	uninstall_idle_process();

	screen_window = gtk_window_new(GTK_WINDOW_DIALOG);
	gtk_object_set_data(GTK_OBJECT(screen_window), "screen_window",
	    screen_window);
	gtk_widget_set_usize(screen_window, 320, 240);
	gtk_container_set_border_width(GTK_CONTAINER(screen_window), 2);
	gtk_window_set_title(GTK_WINDOW(screen_window), "Screen Option");
	gtk_window_set_position(GTK_WINDOW(screen_window), GTK_WIN_POS_CENTER);
	gtk_window_set_modal(GTK_WINDOW(screen_window), TRUE);
	gtk_window_set_default_size(GTK_WINDOW(screen_window), 320, 240);
	gtk_window_set_policy(GTK_WINDOW(screen_window), FALSE, FALSE, FALSE);

	gtk_signal_connect(GTK_OBJECT(screen_window), "destroy",
	    GTK_SIGNAL_FUNC(screen_destroy), NULL);

	vbox2 = gtk_vbox_new(FALSE, 0);
	gtk_widget_ref(vbox2);
	gtk_object_set_data_full(GTK_OBJECT(screen_window), "vbox2", vbox2,
	    (GtkDestroyNotify)gtk_widget_unref);
	gtk_widget_show(vbox2);
	gtk_container_add(GTK_CONTAINER(screen_window), vbox2);

	screen_notebook = gtk_notebook_new();
	gtk_widget_ref(screen_notebook);
	gtk_object_set_data_full(GTK_OBJECT(screen_window), "screen_notebook",
	    screen_notebook, (GtkDestroyNotify)gtk_widget_unref);
	gtk_widget_show(screen_notebook);
	gtk_box_pack_start(GTK_BOX(vbox2), screen_notebook, TRUE, TRUE, 0);
	gtk_widget_set_usize(screen_notebook, 0, 200);

	vbox3 = gtk_vbox_new(TRUE, 0);
	gtk_widget_ref(vbox3);
	gtk_object_set_data_full(GTK_OBJECT(screen_window), "vbox3", vbox3,
	    (GtkDestroyNotify)gtk_widget_unref);
	gtk_widget_show(vbox3);
	gtk_container_add(GTK_CONTAINER(screen_notebook), vbox3);

	lcd_checkbutton = gtk_check_button_new_with_label("Liquid Crystal Display");
	gtk_widget_ref(lcd_checkbutton);
	gtk_object_set_data_full(GTK_OBJECT(screen_window), "lcd_checkbutton",
	    lcd_checkbutton, (GtkDestroyNotify)gtk_widget_unref);
	gtk_widget_show(lcd_checkbutton);
	gtk_box_pack_start(GTK_BOX(vbox3), lcd_checkbutton, FALSE, FALSE, 0);
	if (np2cfg.LCD_MODE & 1) {
		gtk_signal_emit_by_name(GTK_OBJECT(lcd_checkbutton), "clicked");
	}

	reverse_checkbutton = gtk_check_button_new_with_label("reverse");
	gtk_widget_ref(reverse_checkbutton);
	gtk_object_set_data_full(GTK_OBJECT(screen_window),
	    "reverse_checkbutton", reverse_checkbutton,
	    (GtkDestroyNotify)gtk_widget_unref);
	gtk_widget_show(reverse_checkbutton);
	gtk_box_pack_start(GTK_BOX(vbox3), reverse_checkbutton, FALSE, FALSE,0);
	gtk_container_set_border_width(GTK_CONTAINER(reverse_checkbutton), 5);
	if (np2cfg.LCD_MODE & 1) {
		if (np2cfg.LCD_MODE & 2) {
			gtk_signal_emit_by_name(GTK_OBJECT(reverse_checkbutton), "clicked");
		}
	} else {
		gtk_widget_set_sensitive(reverse_checkbutton, 0);
	}
	gtk_signal_connect(GTK_OBJECT(lcd_checkbutton), "clicked",
	    GTK_SIGNAL_FUNC(lcd_checkbutton_clicked),
	    (gpointer)reverse_checkbutton);

	skipline_checkbutton = gtk_check_button_new_with_label("Use skipline revisions");
	gtk_widget_ref(skipline_checkbutton);
	gtk_object_set_data_full(GTK_OBJECT(screen_window),
	    "skipline_checkbutton", skipline_checkbutton,
	    (GtkDestroyNotify)gtk_widget_unref);
	gtk_widget_show(skipline_checkbutton);
	gtk_box_pack_start(GTK_BOX(vbox3), skipline_checkbutton, FALSE, FALSE, 0);
	if (np2cfg.skipline) {
		gtk_signal_emit_by_name(GTK_OBJECT(skipline_checkbutton), "clicked");
	}

	hbox5 = gtk_hbox_new(FALSE, 0);
	gtk_widget_ref(hbox5);
	gtk_object_set_data_full(GTK_OBJECT(screen_window), "hbox5", hbox5,
	    (GtkDestroyNotify)gtk_widget_unref);
	gtk_widget_show(hbox5);
	gtk_box_pack_start(GTK_BOX(vbox3), hbox5, FALSE, FALSE, 0);
	gtk_container_set_border_width(GTK_CONTAINER(hbox5), 2);

	ratio_label = gtk_label_new("Ratio");
	gtk_widget_ref(ratio_label);
	gtk_object_set_data_full(GTK_OBJECT(screen_window), "ratio_label",
	    ratio_label, (GtkDestroyNotify)gtk_widget_unref);
	gtk_widget_show(ratio_label);
	gtk_box_pack_start(GTK_BOX(hbox5), ratio_label, TRUE, TRUE, 0);

	ratio_adj = gtk_adjustment_new(np2cfg.skiplight, 0.0, 255.0, 1.0, 1.0, 0.0);
	ratio_scale = gtk_hscale_new(GTK_ADJUSTMENT(ratio_adj));
	gtk_widget_set_usize(ratio_scale, 200, 0);
	scale_set_default_values(GTK_SCALE(ratio_scale));
	gtk_scale_set_digits(GTK_SCALE(ratio_scale), 0);
	gtk_box_pack_start(GTK_BOX(hbox5), ratio_scale, FALSE, FALSE, 0);
	gtk_widget_show(ratio_scale);

	video_label = gtk_label_new("Video");
	gtk_widget_ref(video_label);
	gtk_object_set_data_full(GTK_OBJECT(screen_window), "video_label",
	    video_label, (GtkDestroyNotify)gtk_widget_unref);
	gtk_widget_show(video_label);
	gtk_notebook_set_tab_label(GTK_NOTEBOOK(screen_notebook),
	    gtk_notebook_get_nth_page(GTK_NOTEBOOK(screen_notebook), 0),
	    video_label);

	vbox4 = gtk_vbox_new(FALSE, 0);
	gtk_widget_ref(vbox4);
	gtk_object_set_data_full(GTK_OBJECT(screen_window), "vbox4", vbox4,
	    (GtkDestroyNotify)gtk_widget_unref);
	gtk_widget_show(vbox4);
	gtk_container_add(GTK_CONTAINER(screen_notebook), vbox4);

	gdc_frame = gtk_frame_new("Graphic Display Controller");
	gtk_widget_ref(gdc_frame);
	gtk_object_set_data_full(GTK_OBJECT(screen_window), "gdc_frame",
	    gdc_frame, (GtkDestroyNotify)gtk_widget_unref);
	gtk_widget_show(gdc_frame);
	gtk_box_pack_start(GTK_BOX(vbox4), gdc_frame, TRUE, TRUE, 0);
	gtk_container_set_border_width(GTK_CONTAINER(gdc_frame), 2);

	hbox6 = gtk_hbox_new(FALSE, 0);
	gtk_widget_ref(hbox6);
	gtk_object_set_data_full(GTK_OBJECT(screen_window), "hbox6", hbox6,
	    (GtkDestroyNotify)gtk_widget_unref);
	gtk_widget_show(hbox6);
	gtk_container_add(GTK_CONTAINER(gdc_frame), hbox6);

	upd72020_radiobutton[0] = gtk_radio_button_new_with_label(hbox6_group, "uPD7220");
	hbox6_group = gtk_radio_button_group(GTK_RADIO_BUTTON(upd72020_radiobutton[0]));
	gtk_widget_ref(upd72020_radiobutton[0]);
	gtk_object_set_data_full(GTK_OBJECT(screen_window),
	    "upd72020_radiobutton[0]", upd72020_radiobutton[0],
	    (GtkDestroyNotify)gtk_widget_unref);
	gtk_widget_show(upd72020_radiobutton[0]);
	gtk_box_pack_start(GTK_BOX(hbox6), upd72020_radiobutton[0], TRUE, FALSE, 0);
	gtk_signal_connect(GTK_OBJECT(upd72020_radiobutton[0]), "clicked",
	    GTK_SIGNAL_FUNC(uPD72020_radiobutton_clicked), (gpointer)0);

	upd72020_radiobutton[1] = gtk_radio_button_new_with_label(hbox6_group, "uPD72020");
	hbox6_group = gtk_radio_button_group(GTK_RADIO_BUTTON(upd72020_radiobutton[1]));
	gtk_widget_ref(upd72020_radiobutton[1]);
	gtk_object_set_data_full(GTK_OBJECT(screen_window),
	    "upd72020_radiobutton[1]", upd72020_radiobutton[1],
	    (GtkDestroyNotify)gtk_widget_unref);
	gtk_widget_show(upd72020_radiobutton[1]);
	gtk_box_pack_start(GTK_BOX(hbox6), upd72020_radiobutton[1], TRUE, FALSE, 0);
	gtk_signal_connect(GTK_OBJECT(upd72020_radiobutton[1]), "clicked",
	    GTK_SIGNAL_FUNC(uPD72020_radiobutton_clicked), (gpointer)1);
	gtk_signal_emit_by_name(GTK_OBJECT(upd72020_radiobutton[np2cfg.uPD72020 ? 1 : 0]), "clicked");

	gc_frame = gtk_frame_new("Graphic Charger");
	gtk_widget_ref(gc_frame);
	gtk_object_set_data_full(GTK_OBJECT(screen_window), "gc_frame",
	    gc_frame, (GtkDestroyNotify)gtk_widget_unref);
	gtk_widget_show(gc_frame);
	gtk_box_pack_start(GTK_BOX(vbox4), gc_frame, TRUE, TRUE, 0);
	gtk_container_set_border_width(GTK_CONTAINER(gc_frame), 2);

	hbox7 = gtk_hbox_new(FALSE, 0);
	gtk_widget_ref(hbox7);
	gtk_object_set_data_full(GTK_OBJECT(screen_window), "hbox7", hbox7,
	(GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show(hbox7);
	gtk_container_add(GTK_CONTAINER(gc_frame), hbox7);

    for (i = 0; i < 4; i++) {
    static char *str[] = { "None", "GRCG", "GRCG+", "EGC" };
	gc_radiobutton[i]=gtk_radio_button_new_with_label(hbox7_group, str[i]);
	hbox7_group=gtk_radio_button_group(GTK_RADIO_BUTTON(gc_radiobutton[i]));
	gtk_widget_ref(gc_radiobutton[i]);
	gtk_object_set_data_full(GTK_OBJECT(screen_window), "gc_radiobutton",
	    gc_radiobutton[i], (GtkDestroyNotify)gtk_widget_unref);
	gtk_widget_show(gc_radiobutton[i]);
	gtk_box_pack_start(GTK_BOX(hbox7), gc_radiobutton[i], TRUE, FALSE, 0);
	gtk_signal_connect(GTK_OBJECT(gc_radiobutton[i]), "clicked",
	    GTK_SIGNAL_FUNC(gc_radiobutton_clicked), (gpointer)i);
    }
	gtk_signal_emit_by_name(GTK_OBJECT(gc_radiobutton[np2cfg.grcg & 3]), "clicked");

	color16_checkbutton = gtk_check_button_new_with_label("Enable 16color (PC-9801-24)");
	gtk_widget_ref(color16_checkbutton);
	gtk_object_set_data_full(GTK_OBJECT(screen_window),
	    "color16_checkbutton", color16_checkbutton,
	    (GtkDestroyNotify)gtk_widget_unref);
	gtk_widget_show(color16_checkbutton);
	gtk_box_pack_start(GTK_BOX(vbox4), color16_checkbutton, FALSE, FALSE,0);
	if (np2cfg.color16) {
		gtk_signal_emit_by_name(GTK_OBJECT(color16_checkbutton), "clicked");
	}

	chip_label = gtk_label_new("Chip");
	gtk_widget_ref(chip_label);
	gtk_object_set_data_full(GTK_OBJECT(screen_window), "chip_label",
	    chip_label, (GtkDestroyNotify)gtk_widget_unref);
	gtk_widget_show(chip_label);
	gtk_notebook_set_tab_label(GTK_NOTEBOOK(screen_notebook),
	    gtk_notebook_get_nth_page(GTK_NOTEBOOK(screen_notebook), 1),
	    chip_label);

	vbox5 = gtk_vbox_new(FALSE, 0);
	gtk_widget_ref(vbox5);
	gtk_object_set_data_full(GTK_OBJECT(screen_window), "vbox5", vbox5,
	    (GtkDestroyNotify)gtk_widget_unref);
	gtk_widget_show(vbox5);
	gtk_container_add(GTK_CONTAINER(screen_notebook), vbox5);

	hbox8 = gtk_hbox_new(FALSE, 0);
	gtk_widget_ref(hbox8);
	gtk_object_set_data_full(GTK_OBJECT(screen_window), "hbox8", hbox8,
	    (GtkDestroyNotify)gtk_widget_unref);
	gtk_widget_show(hbox8);
	gtk_box_pack_start(GTK_BOX(vbox5), hbox8, TRUE, TRUE, 0);

	tram_label = gtk_label_new("TRAM");
	gtk_widget_ref(tram_label);
	gtk_object_set_data_full(GTK_OBJECT(screen_window), "tram_label",
	    tram_label, (GtkDestroyNotify)gtk_widget_unref);
	gtk_widget_show(tram_label);
	gtk_box_pack_start(GTK_BOX(hbox8), tram_label, TRUE, FALSE, 0);
	gtk_widget_set_usize(tram_label, 40, 0);

	tram_adj = gtk_adjustment_new(np2cfg.wait[0], 0.0, 32.0, 1.0, 1.0, 0.0);
	tram_scale = gtk_hscale_new(GTK_ADJUSTMENT(tram_adj));
	gtk_widget_set_usize(tram_scale, 200, 0);
	scale_set_default_values(GTK_SCALE(tram_scale));
	gtk_scale_set_digits(GTK_SCALE(tram_scale), 0);
	gtk_box_pack_start(GTK_BOX(hbox8), tram_scale, TRUE, TRUE, 0);
	gtk_widget_show(tram_scale);

	tramclock_label = gtk_label_new(" clock");
	gtk_widget_ref(tramclock_label);
	gtk_object_set_data_full(GTK_OBJECT(screen_window), "tramclock_label",
	    tramclock_label, (GtkDestroyNotify)gtk_widget_unref);
	gtk_widget_show(tramclock_label);
	gtk_box_pack_start(GTK_BOX(hbox8), tramclock_label, FALSE, FALSE, 0);

	hbox9 = gtk_hbox_new(FALSE, 0);
	gtk_widget_ref(hbox9);
	gtk_object_set_data_full(GTK_OBJECT(screen_window), "hbox9", hbox9,
	(GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show(hbox9);
	gtk_box_pack_start(GTK_BOX(vbox5), hbox9, TRUE, TRUE, 0);

	vram_label = gtk_label_new("VRAM");
	gtk_widget_ref(vram_label);
	gtk_object_set_data_full(GTK_OBJECT(screen_window), "vram_label",
	    vram_label, (GtkDestroyNotify)gtk_widget_unref);
	gtk_widget_show(vram_label);
	gtk_box_pack_start(GTK_BOX(hbox9), vram_label, TRUE, TRUE, 0);
	gtk_widget_set_usize(vram_label, 40, 0);

	vram_adj = gtk_adjustment_new(np2cfg.wait[2], 0.0, 32.0, 1.0, 1.0, 0.0);
	vram_scale = gtk_hscale_new(GTK_ADJUSTMENT(vram_adj));
	gtk_widget_set_usize(vram_scale, 200, 0);
	scale_set_default_values(GTK_SCALE(vram_scale));
	gtk_scale_set_digits(GTK_SCALE(vram_scale), 0);
	gtk_box_pack_start(GTK_BOX(hbox9), vram_scale, TRUE, TRUE, 0);
	gtk_widget_show(vram_scale);

	vramclock_label = gtk_label_new(" clock");
	gtk_widget_ref(vramclock_label);
	gtk_object_set_data_full(GTK_OBJECT(screen_window), "vramclock_label",
	    vramclock_label,
	(GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show(vramclock_label);
	gtk_box_pack_start(GTK_BOX(hbox9), vramclock_label, FALSE, FALSE, 0);

	hbox10 = gtk_hbox_new(FALSE, 0);
	gtk_widget_ref(hbox10);
	gtk_object_set_data_full(GTK_OBJECT(screen_window), "hbox10", hbox10,
	    (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show(hbox10);
	gtk_box_pack_start(GTK_BOX(vbox5), hbox10, TRUE, TRUE, 0);

	grcg_label = gtk_label_new("GRCG");
	gtk_widget_ref(grcg_label);
	gtk_object_set_data_full(GTK_OBJECT(screen_window), "grcg_label",
	    grcg_label, (GtkDestroyNotify)gtk_widget_unref);
	gtk_widget_show(grcg_label);
	gtk_box_pack_start(GTK_BOX(hbox10), grcg_label, TRUE, TRUE, 0);
	gtk_widget_set_usize(grcg_label, 44, 0);

	grcg_adj = gtk_adjustment_new(np2cfg.wait[4], 0.0, 32.0, 1.0, 1.0, 0.0);
	grcg_scale = gtk_hscale_new(GTK_ADJUSTMENT(grcg_adj));
	gtk_widget_set_usize(grcg_scale, 200, 0);
	scale_set_default_values(GTK_SCALE(grcg_scale));
	gtk_scale_set_digits(GTK_SCALE(grcg_scale), 0);
	gtk_box_pack_start(GTK_BOX(hbox10), grcg_scale, TRUE, TRUE, 0);
	gtk_widget_show(grcg_scale);

	grcgclock_label = gtk_label_new(" clock");
	gtk_widget_ref(grcgclock_label);
	gtk_object_set_data_full(GTK_OBJECT(screen_window), "grcgclock_label",
	    grcgclock_label, (GtkDestroyNotify)gtk_widget_unref);
	gtk_widget_show(grcgclock_label);
	gtk_box_pack_start(GTK_BOX(hbox10), grcgclock_label, FALSE, FALSE, 0);

	timing_label = gtk_label_new("Timing");
	gtk_widget_ref(timing_label);
	gtk_object_set_data_full(GTK_OBJECT(screen_window), "timing_label",
	    timing_label, (GtkDestroyNotify)gtk_widget_unref);
	gtk_widget_show(timing_label);
	gtk_notebook_set_tab_label(GTK_NOTEBOOK(screen_notebook),
	    gtk_notebook_get_nth_page(GTK_NOTEBOOK(screen_notebook), 2),
	    timing_label);

	hbox1 = gtk_hbox_new(FALSE, 0);
	gtk_widget_ref(hbox1);
	gtk_object_set_data_full(GTK_OBJECT(screen_window), "hbox1", hbox1,
	    (GtkDestroyNotify)gtk_widget_unref);
	gtk_widget_show(hbox1);
	gtk_box_pack_start(GTK_BOX(vbox2), hbox1, TRUE, TRUE, 0);

	null_label = gtk_label_new("");
	gtk_widget_ref(null_label);
	gtk_object_set_data_full(GTK_OBJECT(screen_window), "null_label",
	    null_label, (GtkDestroyNotify)gtk_widget_unref);
	gtk_box_pack_start(GTK_BOX(hbox1), null_label, TRUE, TRUE, 0);
	gtk_widget_set_sensitive(null_label, FALSE);

	ok_button = gtk_button_new_with_label("OK");
	gtk_widget_ref(ok_button);
	gtk_object_set_data_full(GTK_OBJECT(screen_window), "ok_button",
	    ok_button, (GtkDestroyNotify)gtk_widget_unref);
	gtk_widget_show(ok_button);
	gtk_box_pack_start(GTK_BOX(hbox1), ok_button, FALSE, FALSE, 0);
	gtk_widget_set_usize(ok_button, 96, 48);
	gtk_container_set_border_width(GTK_CONTAINER(ok_button), 3);
	gtk_signal_connect(GTK_OBJECT(ok_button), "clicked",
	    GTK_SIGNAL_FUNC(ok_button_clicked), (gpointer)screen_window);
	GTK_WIDGET_SET_FLAGS(ok_button, GTK_HAS_DEFAULT);
	GTK_WIDGET_SET_FLAGS(ok_button, GTK_CAN_DEFAULT);
	gtk_widget_grab_default(ok_button);

	cancel_button = gtk_button_new_with_label("Cancel");
	gtk_widget_ref(cancel_button);
	gtk_object_set_data_full(GTK_OBJECT(screen_window), "cancel_button",
	    cancel_button, (GtkDestroyNotify)gtk_widget_unref);
	gtk_widget_show(cancel_button);
	gtk_box_pack_start(GTK_BOX(hbox1), cancel_button, FALSE, FALSE, 0);
	gtk_widget_set_usize(cancel_button, 96, 48);
	gtk_container_set_border_width(GTK_CONTAINER(cancel_button), 5);
	gtk_signal_connect_object(GTK_OBJECT(cancel_button), "clicked",
	    GTK_SIGNAL_FUNC(gtk_widget_destroy), GTK_OBJECT(screen_window));

	gtk_widget_show_all(screen_window);
}
