/*	$Id: gtkwrapper.c,v 1.3 2002/03/08 16:27:34 nonaka Exp $	*/

/*
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgment:
 *      This product includes software developed by NONAKA Kimihiro.
 * 4. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include "compiler.h"

#include <sys/ipc.h>
#include <sys/shm.h>
#include <sys/types.h>

#include <errno.h>
#include <stdio.h>
#include <stdlib.h>

#include <X11/Xlib.h>
#include <X11/extensions/XShm.h>

#include <gtk/gtk.h>
#include <gdk/gdkprivate.h>

#include "xnp2.h"

void
gdk_window_set_pointer(GdkWindow *window, gint x, gint y)
{ 
	GdkWindowPrivate *private;

	if (!window)
		window = (GdkWindow *)&gdk_root_parent;

	private = (GdkWindowPrivate *)window;
	if (private->destroyed)
		return;

	XWarpPointer(private->xdisplay, None, private->xwindow,
	    0, 0, 0, 0, x, y);
}

int
is_32bpp(GdkWindow *w)
{
	GdkWindowPrivate *private;
	XPixmapFormatValues *format;
	int nbit = 0;
	int count;
	int i;

	if (w == NULL)
		w = (GdkWindow *)&gdk_root_parent;
	private = (GdkWindowPrivate *)w;
	if (private->destroyed)
		return 0;

	format = XListPixmapFormats(private->xdisplay, &count);
	if (format != 0) {
		for (i = 0; i < count; i++) {
			if (format[i].depth != 24)
				continue;

			if (format[i].bits_per_pixel == 32) {
				nbit = 32;
			} else {
				nbit = 24;
			}
			break;
		}
		if (i == count) {
			XFree(format);
			fprintf(stderr, "24bpp depth not support?\n");
			return 0;
		}
		XFree(format);
	} else {
		fprintf(stderr, "Can't get PixmapFormats.\n");
		return 0;
	}

	return (nbit == 32) ? 1 : 0;
}
