#include	"compiler.h"

#include	"np2.h"
#include	"i286.h"
#include	"pc_core.h"
#include	"iobridge.h"
#include	"gdc.h"
#include	"gdc_cmd.h"
#include	"gdc_sub.h"
#include	"crtc.h"
#include	"palettes.h"
#include	"nevent.h"
#include	"memory.h"
#include	"vram.h"
#include	"trace.h"

#define	SEARHC_SYNC


	GDC_TABLE	gdc;
extern BYTE screenupdate;

#ifdef __cplusplus
extern "C" {
#endif
	GDCS_TABLE	gdcs;
#ifdef __cplusplus
}
#endif

static const BYTE defaultdegpal[4] = {0x04, 0x15, 0x26, 0x37};
static const XSTR defaultsync[8] = {
				0x10, 0x4e, 0x07, 0x25, 0x07, 0x07, 0x90, 0x65};


void gdc_setdegitalpal(int color, BYTE value) {

	if (color & 4) {
		color &= 3;
		value &= 0x0f;
		if ((gdc.degpal[color] ^ value) & 0x07) {
			gdcs.palchange = GDCSCRN_REDRAW;
		}
		gdc.degpal[color] &= 0xf0;
		gdc.degpal[color] |= value;
	}
	else {
		color &= 3;
		value <<= 4;
		if ((gdc.degpal[color] ^ value) & 0x70) {
			gdcs.palchange = GDCSCRN_REDRAW;
		}
		gdc.degpal[color] &= 0x0f;
		gdc.degpal[color] |= value;
	}
}

void gdc_setanalogpal(int color, int rgb, BYTE value) {

	if ((gdc.anapal[color].b[rgb] ^ value) & 0x0f) {
		gdcs.palchange = GDCSCRN_REDRAW;
		if ((palevent.events < PALEVENTMAX) && (!gdc.vsync)) {
			palevent.event[palevent.events].clock = 
							nevent.item[NEVENT_FLAMES].clock -
									(nevent.baseclock - nevent.remainclock);
			palevent.event[palevent.events].color = (color << 2) | rgb;
			palevent.event[palevent.events].value = value;
			palevent.events++;
		}
	}
	gdc.anapal[color].b[rgb] = value;
}

void gdc_setdegpalpack(int color, BYTE value) {

	if ((gdc.degpal[color] ^ value) & 0x77) {
		gdcs.palchange = GDCSCRN_REDRAW;
	}
	gdc.degpal[color] = value;
}


void gdc_paletteinit(void) {

	BYTE	c;

	CopyMemory(gdc.degpal, defaultdegpal, 4);

	for (c=0; c<8; c++) {
		gdc.anapal[c+0].p.b = ((c&1)?0x0f:0);
		gdc.anapal[c+8].p.b = ((c&1)?0x0a:0);

		gdc.anapal[c+0].p.r = ((c&2)?0x0f:0);
		gdc.anapal[c+8].p.r = ((c&2)?0x0a:0);

		gdc.anapal[c+0].p.g = ((c&4)?0x0f:0);
		gdc.anapal[c+8].p.g = ((c&4)?0x0a:0);
	}
	gdc.anapal[8].p.b = 0x07;
	gdc.anapal[8].p.r = 0x07;
	gdc.anapal[8].p.g = 0x07;

	gdcs.palchange = GDCSCRN_REDRAW;
}

// --------------------------------------------------------------------------

void gdc_init(void) {

	ZeroMemory(&gdc, sizeof(gdc));
	ZeroMemory(&gdcs, sizeof(gdcs));

	gdc.mode1 = 0x98;
	gdc.clock = 0;

	gdc.m.para[GDC_CSRFORM + 0] = 0x0f;
	gdc.m.para[GDC_CSRFORM + 1] = 0xc0;
	gdc.m.para[GDC_CSRFORM + 2] = 0x7b;

	CopyMemory(gdc.m.para + GDC_SYNC, defaultsync, sizeof(defaultsync));
	CopyMemory(gdc.s.para + GDC_SYNC, defaultsync, sizeof(defaultsync));

	gdc.s.para[GDC_CSRFORM] = 1;
	gdc.m.para[GDC_PITCH] = 80;					// ver0.26
	gdc.s.para[GDC_PITCH] = 40;

	gdc_paletteinit();

	gdcs.textdisp = GDCSCRN_ENABLE | GDCSCRN_ALLDRAW2 | GDCSCRN_EXT;
	gdcs.grphdisp = GDCSCRN_ALLDRAW2 | GDCSCRN_EXT;
	gdc.display = (np2cfg.color16 & 1) << 1;
	gdc.bitac = 0xff;
}

I286_OUT gdc_master_w(WORD port, BYTE value) {

	BYTE	bit;

	switch(port & 0x0f) {
		case 0x00:
			TRACEOUT(port, value);
			if (gdc.m.cnt < GDCCMD_MAX) {
				gdc.m.fifo[gdc.m.cnt++] = value;
			}
			break;

		case 0x02:
			TRACEOUT(port, value);
			if (gdc.m.cnt < GDCCMD_MAX) {
				gdc.m.fifo[gdc.m.cnt++] = 0x100 | value;
			}
			gdc_work(GDCWORK_MASTER);
			break;

		case 0x04:
			gdc.vsyncint = 1;
			break;

		case 0x08:
			if (!(value & 0xf0)) {
				bit = 1 << ((value >> 1) & 7);
				if (value & 1) {
					gdc.mode1 |= bit;
				}
				else {
					gdc.mode1 &= ~bit;
				}
				if (bit & (0x01 | 0x04 | 0x10)) {
					gdcs.grphdisp |= GDCSCRN_ALLDRAW2;
				}
				else if (bit == 0x02) {								// ver0.28
					gdcs.grphdisp |= GDCSCRN_ALLDRAW2;
					gdcs.palchange = GDCSCRN_REDRAW;
				}
				else if (bit == 0x20) {
					gdc_restorekacmode();
				}
				else if (bit == 0x80) {
					screenupdate |= 2;
				}
				gdcs.msw_accessable = gdc.mode1 & 0x40;
			}
			break;

		case 0x0a:
//			TRACEOUT(port, value);
			if (!(value & 0xf8)) {
				bit = (value >> 1) & 3;
				value &= 1;
				if (!(((gdc.mode2 >> bit) ^ value) & 1)) {
					break;
				}
				gdc.mode2 ^= (1 << bit);
				switch(bit) {
					case 0:
						if (gdc.display & 2) {
							gdc.analog = value;
							gdcs.palchange = GDCSCRN_REDRAW;
							vramop.operate &= VOP_ANALOGMASK;
							vramop.operate |= value << 4;
							i286_vram_dispatch(vramop.operate);
						}
						break;
					case 2:
						if ((gdc.mode2 & 0x08) && (grcg.chip == 3)) {
							vramop.operate &= VOP_EGCMASK;
							vramop.operate |= value << 1;
							i286_vram_dispatch(vramop.operate);
						}
						break;
				}
				gdcs.mode2 = gdc.mode2;
			}
			else {
				switch(value) {
					case 0x40:
						gdc.display &= ~1;
						gdcs.textdisp |= GDCSCRN_EXT;
						break;
					case 0x41:
						gdc.display |= 1;
						gdcs.textdisp |= GDCSCRN_EXT;
						break;
					case 0x82:
						gdc.clock &= ~1;
						gdcs.grphdisp |= GDCSCRN_EXT;
						break;
					case 0x83:
						gdc.clock |= 1;
						gdcs.grphdisp |= GDCSCRN_EXT;
						break;
					case 0x84:
						gdc.clock &= ~2;
						gdcs.grphdisp |= GDCSCRN_EXT;
						break;
					case 0x85:
						gdc.clock |= 2;
						gdcs.grphdisp |= GDCSCRN_EXT;
						break;
				}
			}
			break;
	}
}


I286_INB gdc_master_r(WORD port) {

	BYTE	ret;

	switch(port & 0x0f) {
		case 0x00:
			ret = 0x80 | gdc.vsync;		// | m_drawing;
			if (!gdc.vsync) {
				long remain = nevent_getremain(NEVENT_FLAMES);
				if (remain >= 0) {
//					remain /= pc.multiple;							// ver0.28
					if ((remain % pc.raster) < (pc.hsync)) {
						ret |= 0x40;
					}
				}
			}
			if (gdc.m.snd) {
				ret |= 0x01;
			}
			if (gdc.m.cnt >= GDCCMD_MAX) {
				ret |= 0x02;
			}
			if (!gdc.m.cnt) {
				ret |= 0x04;
			}
			else {
				gdc_work(GDCWORK_MASTER);
			}
#ifdef SEARHC_SYNC
			if ((i286s.inport) && (nevent.remainclock >= 5)) {
				WORD	jadr = 0xfa74;
				WORD	memv;
				memv = i286_memoryread_w(i286s.inport);
				while((memv == 0x00eb) || (memv == 0x5fe6)) {
					jadr -= 0x200;
					i286s.inport += 2;
					memv = i286_memoryread_w(i286s.inport);
				}
				if ((memv == 0x20a8) || (memv == 0x2024)) {
					memv = i286_memoryread_w(i286s.inport + 2);
					if (memv == jadr) {					// je
						if (!gdc.vsync) {
							nevent.remainclock = -1;
						}
					}
					else if (memv == (jadr + 1)) {		// jne
						if (gdc.vsync) {
							nevent.remainclock = -1;
						}
					}
				}
			}
#endif
			return(ret);

		case 0x02:
			if (gdc.m.snd) {
				gdc.m.snd--;
				return(gdc.m.para[gdc.m.ptr++]);
			}
			break;

		case 0x08:
			return(gdc.mode1);

		case 0x0a:
			return(gdc.mode2);
	}
	return(0xff);
}


I286_OUT gdc_slave_w(WORD port, BYTE value) {

	switch(port & 0x0f) {
		case 0x00:
//			TRACEOUT(port, value);
			if (gdc.s.cnt < GDCCMD_MAX) {
				gdc.s.fifo[gdc.s.cnt++] = value;
			}
			break;

		case 0x02:
//			TRACEOUT(port, value);
			if (gdc.s.cnt < GDCCMD_MAX) {
				gdc.s.fifo[gdc.s.cnt++] = 0x100 | value;
			}
			gdc_work(GDCWORK_SLAVE);
			break;

		case 0x04:
			if ((gdcs.disp ^ value) & 1) {
				gdcs.disp = value & 1;
				screenupdate |= 2;
			}
			break;

		case 0x06:
			if ((gdcs.access ^ value) & 1) {
				gdcs.access = value & 1;
				vramop.operate &= VOP_ACCESSMASK;
				vramop.operate |= gdcs.access;
				i286_vram_dispatch(vramop.operate);
			}
			break;

		case 0x08:
//			TRACEOUT(port, value);
			if (gdc.analog) {
				gdc.palnum = value & 0x0f;
			}
			else {
				gdc_setdegpalpack(3, value);
			}
			break;

		case 0x0a:
//			TRACEOUT(port, value);
			if (gdc.analog) {
				gdc_setanalogpal(gdc.palnum, GDC_GREEN, value);
			}
			else {
				gdc_setdegpalpack(1, value);
			}
			break;

		case 0x0c:
//			TRACEOUT(port, value);
			if (gdc.analog) {
				gdc_setanalogpal(gdc.palnum, GDC_RED, value);
			}
			else {
				gdc_setdegpalpack(2, value);
			}
			break;

		case 0x0e:
//			TRACEOUT(port, value);
			if (gdc.analog) {
				gdc_setanalogpal(gdc.palnum, GDC_BLUE, value);
			}
			else {
				gdc_setdegpalpack(0, value);
			}
			break;
	}
}


I286_INB gdc_slave_r(WORD port) {

	BYTE	ret;

//	TRACEIN(port);

	switch(port & 0x0f) {
		case 0x00:
			ret = 0x80 | gdc.vsync | gdc.s_drawing;
			if (!gdc.vsync) {
				long remain = nevent_getremain(NEVENT_FLAMES);
				if (remain >= 0) {
//					remain /= pc.multiple;							// ver0.28
					if ((remain % pc.raster) < (pc.hsync)) {
						ret |= 0x40;
					}
				}
			}
			if (gdc.s.snd) {
				ret |= 0x01;
			}
			if (gdc.s.cnt >= GDCCMD_MAX) {
				ret |= 0x02;
			}
			if (!gdc.s.cnt) {
				ret |= 0x04;
			}
			else {
				gdc_work(GDCWORK_SLAVE);
			}
#ifdef SEARHC_SYNC
			if ((i286s.inport) && (nevent.remainclock >= 5)) {
				WORD	jadr = 0xfa74;
				WORD	memv;
				memv = i286_memoryread_w(i286s.inport);
				while((memv == 0x00eb) || (memv == 0x5fe6)) {
					jadr -= 0x200;
					i286s.inport += 2;
					memv = i286_memoryread_w(i286s.inport);
				}
				if ((memv == 0x20a8) || (memv == 0x2024)) {
					memv = i286_memoryread_w(i286s.inport + 2);
					if (memv == jadr) {					// je
						if (!gdc.vsync) {
							nevent.remainclock = -1;
						}
					}
					else if (memv == (jadr + 1)) {		// jne
						if (gdc.vsync) {
							nevent.remainclock = -1;
						}
					}
				}
			}
#endif
//TRACEINA(port, ret);
			return(ret);

		case 0x02:
			if (gdc.s.snd) {
				gdc.s.snd--;
				return(gdc.s.para[gdc.s.ptr++]);
			}
			break;

		case 0x04:
			return(gdcs.disp);

		case 0x06:
			return(gdcs.access);
	}
	return(0xff);
}

static void vectdraw(void) {

	if (gdc.s.para[GDC_VECTW] & 0x8) {
		gdcsub_line(GETDWORD(gdc.s.para + GDC_CSRW), gdc.s.para + GDC_VECTW,
					GETWORD(gdc.s.para + GDC_TEXTW), gdc.s.para[GDC_WRITE]);
	}
	if (gdc.s.para[GDC_VECTW] & 0x20) {
		gdcsub_circle(GETDWORD(gdc.s.para + GDC_CSRW), gdc.s.para + GDC_VECTW,
					GETWORD(gdc.s.para + GDC_TEXTW), gdc.s.para[GDC_WRITE]);
	}
	if (gdc.s.para[GDC_VECTW] & 0x40) {
		gdcsub_box(GETDWORD(gdc.s.para + GDC_CSRW), gdc.s.para + GDC_VECTW,
					GETWORD(gdc.s.para + GDC_TEXTW), gdc.s.para[GDC_WRITE]);
	}
}

void gdc_work(int id) {

	WORD			i;
	BYTE			data;
	GDCDATA_TABLE	*item;
	BYTE			*dispflag;

	item = ((id==GDCWORK_MASTER)?(&gdc.m):(&gdc.s));
	dispflag = ((id==GDCWORK_MASTER)?(&gdcs.textdisp):(&gdcs.grphdisp));

	for (i=0; i<item->cnt; i++) {
		data = item->fifo[i] & 0xff;
		if (item->fifo[i] & 0xff00) {
			if ((data & 0x60) == 0x20) {
				item->para[GDC_WRITE] = data;
			}
			switch(data) {
				case CMD_START_:
				case CMD_START:
				case CMD_SYNC_ON:
					(*dispflag) |= GDCSCRN_ENABLE | GDCSCRN_ALLDRAW2;
					screenupdate |= 2;								// ver0.28
					break;
				case CMD_STOP:
				case CMD_SYNC_OFF:
					(*dispflag) &= (~GDCSCRN_ENABLE);
//					(*dispflag) |= GDCSCRN_ALLDRAW2;
					screenupdate |= 2;								// ver0.28
					break;
				case CMD_VECTE:
					if (id != GDCWORK_MASTER) {
						vectdraw();
					}
					break;
			}
			item->ptr = gdc_cmd[data].pos;
			item->rcv = gdc_cmd[data].outdatas;
			item->snd = gdc_cmd[data].indatas;
			if (item->snd) {
				switch(item->ptr) {
					case GDC_CSRR:
						item->para[GDC_CSRR+0] = item->para[GDC_CSRW+0];
						item->para[GDC_CSRR+1] = item->para[GDC_CSRW+1];
						item->para[GDC_CSRR+2] = item->para[GDC_CSRW+2] & 3;
						item->para[GDC_CSRR+3] = 0;
						item->para[GDC_CSRR+4] = 0;
						break;
				}
			}
		}
		else if (item->rcv) {
			(item->rcv)--;
			if (item->para[item->ptr] != data) {
				item->para[item->ptr] = data;
				(*dispflag) |= gdc_dirtyflag[id][item->ptr];
			}
			(item->ptr)++;
		}
	}
	item->cnt = 0;
}




// BIOSȤϮä˥ꥻå
void gdc_reset(GDCDATA_TABLE *item) {

	item->rcv = 0;
	item->snd = 0;
}


void gdc_restorekacmode(void) {

	BYTE	bit;

	bit = (((!np2cfg.uPD72020) && (gdc.mode1 & 0x20))?0x00:0xff);
	if (gdc.bitac != bit) {
		gdc.bitac = bit;
		gdcs.textdisp |= GDCSCRN_ALLDRAW2;
	}
}
