#include	"compiler.h"

#include	"np2.h"
#include	"np2pc98.h"
#include	"pc_core.h"
#include	"palettes.h"
#include	"layer.h"


// 24bitߡ
static void lay24(void) {

	BYTE	*d = layer.surface;
	int		x;
	BYTE	*r;
	BYTE	c;

	do {
		if (renewal_line[layer.y] & 0x80) {
			renewal_line[layer.y] &= 0x7f;
			for (r=d, x=0; x<layer.x; x++) {
				c = NP2PAL_TEXT2;
				r[RGB_R] = np2_pal32[c].p.r;
				r[RGB_G] = np2_pal32[c].p.g;
				r[RGB_B] = np2_pal32[c].p.b;
				r += RGB24_STEP;
			}
		}
		d += layer.width;
	} while(++layer.y < layer.maxy);

	layer.surface = d;
}


//	text or grph 1ץ졼 / 24bit
static void np2layer24_1(void) {

	BYTE	*d = layer.surface;
	BYTE	*s = layer.src1scrn;
	int		x;
	BYTE	*r;
	BYTE	c;

	do {
		if (renewal_line[layer.y] & layer.bit) {
			renewal_line[layer.y] &= ~layer.bit;
			for (r=d, x=0; x<layer.x; x++) {
				c = s[x] + NP2PAL_GRPH;
				r[RGB_R] = np2_pal32[c].p.r;
				r[RGB_G] = np2_pal32[c].p.g;
				r[RGB_B] = np2_pal32[c].p.b;
				r += RGB24_STEP;
			}
		}
		s += SURFACE_WIDTH;
		d += layer.width;
	} while(++layer.y < layer.maxy);

	layer.surface = d;
	layer.src1scrn = s;
}


//	text + grph / 24bit
static void np2layer24_tg(void) {

	BYTE	*d = layer.surface;
	BYTE	*s1 = layer.src1scrn;
	BYTE	*s2 = layer.src2scrn;
	int		x;
	BYTE	*r;
	BYTE	c;

	do {
		if (renewal_line[layer.y] & layer.bit) {
			renewal_line[layer.y] &= ~layer.bit;
			for (r=d, x=0; x<layer.x; x++) {
				c = s1[x] + s2[x] + NP2PAL_GRPH;
				r[RGB_R] = np2_pal32[c].p.r;
				r[RGB_G] = np2_pal32[c].p.g;
				r[RGB_B] = np2_pal32[c].p.b;
				r += RGB24_STEP;
			}
		}
		s1 += SURFACE_WIDTH;
		s2 += SURFACE_WIDTH;
		d += layer.width;
	} while(++layer.y < layer.maxy);

	layer.surface = d;
	layer.src1scrn = s1;
	layer.src2scrn = s2;
}


//	text + (grph:skipline) / 24bit
static void np2layer24_t_skip(void) {

	BYTE	*d = layer.surface;
	BYTE	*s = layer.src1scrn;
	int		x;
	BYTE	*r;
	BYTE	c;

	do {
		if (renewal_line[layer.y] & layer.bit) {
			renewal_line[layer.y] &= ~layer.bit;
			for (r=d, x=0; x<layer.x; x++) {
				c = s[x] + NP2PAL_GRPH;
				r[RGB_R] = np2_pal32[c].p.r;
				r[RGB_G] = np2_pal32[c].p.g;
				r[RGB_B] = np2_pal32[c].p.b;
				r += RGB24_STEP;
			}
		}
		s += SURFACE_WIDTH;
		d += layer.width;
		layer.y++;
		if (renewal_line[layer.y] & layer.bit) {
			renewal_line[layer.y] &= ~layer.bit;
			for (r=d, x=0; x<layer.x; x++) {
				c = (s[x] >> 4) + NP2PAL_TEXT;
				r[RGB_R] = np2_pal32[c].p.r;
				r[RGB_G] = np2_pal32[c].p.g;
				r[RGB_B] = np2_pal32[c].p.b;
				r += RGB24_STEP;
			}
		}
		s += SURFACE_WIDTH;
		d += layer.width;
	} while(++layer.y < layer.maxy);

	layer.surface = d;
	layer.src1scrn = s;
}


//	grph:skipline / 24bit
static void np2layer24_g_skip(void) {

	BYTE	*d = layer.surface;
	BYTE	*s = layer.src1scrn;
	int		x;
	BYTE	*r;
	BYTE	c;

	do {
		if (renewal_line[layer.y] & layer.bit) {
			renewal_line[layer.y] &= ~layer.bit;
			for (r=d, x=0; x<layer.x; x++) {
				c = s[x] + NP2PAL_GRPH;
				r[RGB_R] = np2_pal32[c].p.r;
				r[RGB_G] = np2_pal32[c].p.g;
				r[RGB_B] = np2_pal32[c].p.b;
				r += RGB24_STEP;
			}
		}
		s += SURFACE_WIDTH;
		d += layer.width;
		layer.y++;
		if (renewal_line[layer.y] & layer.bit) {
			renewal_line[layer.y] &= ~layer.bit;
			for (r=d, x=0; x<layer.x; x++) {
				r[RGB_R] = np2_pal32[NP2PAL_TEXT].p.r;
				r[RGB_G] = np2_pal32[NP2PAL_TEXT].p.g;
				r[RGB_B] = np2_pal32[NP2PAL_TEXT].p.b;
				r += RGB24_STEP;
			}
		}
		s += SURFACE_WIDTH;
		d += layer.width;
	} while(++layer.y < layer.maxy);

	layer.surface = d;
	layer.src1scrn = s;
}


//	text + grph:skipline / 24bit
static void np2layer24_tg_skip(void) {

	BYTE	*d = layer.surface;
	BYTE	*s1 = layer.src1scrn;
	BYTE	*s2 = layer.src2scrn;
	int		x;
	BYTE	*r;
	BYTE	c;

	do {
		if (renewal_line[layer.y] & layer.bit) {
			renewal_line[layer.y] &= ~layer.bit;
			for (r=d, x=0; x<layer.x; x++) {
				c = s1[x] + s2[x] + NP2PAL_GRPH;
				r[RGB_R] = np2_pal32[c].p.r;
				r[RGB_G] = np2_pal32[c].p.g;
				r[RGB_B] = np2_pal32[c].p.b;
				r += RGB24_STEP;
			}
		}
		s2 += SURFACE_WIDTH;
		d += layer.width;
		layer.y++;
		if (renewal_line[layer.y] & layer.bit) {
			renewal_line[layer.y] &= ~layer.bit;
			for (r=d, x=0; x<layer.x; x++) {
				c = (s2[x] >> 4) + NP2PAL_TEXT;
				r[RGB_R] = np2_pal32[c].p.r;
				r[RGB_G] = np2_pal32[c].p.g;
				r[RGB_B] = np2_pal32[c].p.b;
				r += RGB24_STEP;
			}
		}
		s1 += (SURFACE_WIDTH * 2);
		s2 += SURFACE_WIDTH;
		d += layer.width;
	} while(++layer.y < layer.maxy);

	layer.surface = d;
	layer.src1scrn = s1;
	layer.src2scrn = s2;
}


// --- with extend skipline

//	grph + extend skipline / 24bit
static void np2layer24_g_ext(void) {

	BYTE	*d = layer.surface;
	BYTE	*s = layer.src1scrn;
	int		x;
	BYTE	*r;
	BYTE	c;

	do {
		if (renewal_line[layer.y] & layer.bit) {
			renewal_line[layer.y] &= ~layer.bit;
			renewal_line[layer.y + 1] |= 0x80;				// next line draw
			for (r=d, x=0; x<layer.x; x++) {
				c = s[x] + NP2PAL_GRPH;
				r[RGB_R] = np2_pal32[c].p.r;
				r[RGB_G] = np2_pal32[c].p.g;
				r[RGB_B] = np2_pal32[c].p.b;
				r += RGB24_STEP;
			}
		}
		d += layer.width;
		layer.y++;
		if (renewal_line[layer.y] & layer.bit) {
			renewal_line[layer.y] &= ~layer.bit;
			for (r=d, x=0; x<layer.x; x++) {
				c = s[x] + NP2PAL_SKIP;
				r[RGB_R] = np2_pal32[c].p.r;
				r[RGB_G] = np2_pal32[c].p.g;
				r[RGB_B] = np2_pal32[c].p.b;
				r += RGB24_STEP;
			}
		}
		s += (SURFACE_WIDTH * 2);
		d += layer.width;
	} while(++layer.y < layer.maxy);

	layer.surface = d;
	layer.src1scrn = s;
}

//	text + grph + extend skipline / 24bit
static void np2layer24_tg_ext(void) {

	BYTE	*d = layer.surface;
	BYTE	*s1 = layer.src1scrn;
	BYTE	*s2 = layer.src2scrn;
	int		x;
	BYTE	*r;
	BYTE	c;

	do {
		if (renewal_line[layer.y] & layer.bit) {
			renewal_line[layer.y] &= ~layer.bit;
			renewal_line[layer.y + 1] |= 0x80;				// next line draw
			for (r=d, x=0; x<layer.x; x++) {
				c = s1[x] + s2[x] + NP2PAL_GRPH;
				r[RGB_R] = np2_pal32[c].p.r;
				r[RGB_G] = np2_pal32[c].p.g;
				r[RGB_B] = np2_pal32[c].p.b;
				r += RGB24_STEP;
			}
		}
		s2 += SURFACE_WIDTH;
		d += layer.width;
		layer.y++;
		if (renewal_line[layer.y] & layer.bit) {
			renewal_line[layer.y] &= ~layer.bit;
			for (r=d, x=0; x<layer.x; x++) {
				c = s2[x] >> 4;
				if (!c) {
					c = s1[x] + NP2PALS_TXT;
				}
				c += NP2PAL_TEXT;
				r[RGB_R] = np2_pal32[c].p.r;
				r[RGB_G] = np2_pal32[c].p.g;
				r[RGB_B] = np2_pal32[c].p.b;
				r += RGB24_STEP;
			}
		}
		s1 += (SURFACE_WIDTH * 2);
		s2 += SURFACE_WIDTH;
		d += layer.width;
	} while(++layer.y < layer.maxy);

	layer.surface = d;
	layer.src1scrn = s1;
	layer.src2scrn = s2;
}


// ----------------------------------------- support normal display

#ifdef SUPPORT_NORMALDISP

//	24bitߡ
static void lay24n(void) {

	BYTE	*d = layer.surface;
	int		x;
	BYTE	*r;
	BYTE	c;

	do {
		if (renewal_line[layer.y] & 0x80) {
			renewal_line[layer.y] &= 0x7f;
			for (r=d, x=0; x<=layer.x; x++) {
				c = NP2PAL_TEXT2;
				r[RGB_R] = np2_pal32[c].p.r;
				r[RGB_G] = np2_pal32[c].p.g;
				r[RGB_B] = np2_pal32[c].p.b;
				r += RGB24_STEP;
			}
		}
		d += layer.width;
	} while(++layer.y < layer.maxy);
	layer.surface = d;
}


//	text / 24bit
static void np2layer24n_t(void) {

	BYTE	*d = layer.surface;
	BYTE	*s = layer.src1scrn;
	int		x;
	BYTE	*r;
	BYTE	c;

	do {
		if (renewal_line[layer.y] & layer.bit) {
			renewal_line[layer.y] &= ~layer.bit;
			for (r=d, x=0; x<layer.x; x++) {
				c = s[x] + NP2PAL_GRPH;
				r[RGB_R] = np2_pal32[c].p.r;
				r[RGB_G] = np2_pal32[c].p.g;
				r[RGB_B] = np2_pal32[c].p.b;
				r += RGB24_STEP;
			}
			r[RGB_R] = np2_pal32[NP2PAL_GRPH].p.r;
			r[RGB_G] = np2_pal32[NP2PAL_GRPH].p.g;
			r[RGB_B] = np2_pal32[NP2PAL_GRPH].p.b;
		}
		s += SURFACE_WIDTH;
		d += layer.width;
	} while(++layer.y < layer.maxy);

	layer.surface = d;
	layer.src1scrn = s;
}

//	grph / 24bit
static void np2layer24n_g(void) {

	BYTE	*d = layer.surface;
	BYTE	*s = layer.src1scrn;
	int		x;
	BYTE	*r;
	BYTE	c;

	do {
		if (renewal_line[layer.y] & layer.bit) {
			renewal_line[layer.y] &= ~layer.bit;
			r = d;
			r[RGB_R] = 0;
			r[RGB_G] = 0;
			r[RGB_B] = 0;
			for (x=0; x<layer.x; x++) {
				c = s[x] + NP2PAL_GRPH;
				r += RGB24_STEP;
				r[RGB_R] = np2_pal32[c].p.r;
				r[RGB_G] = np2_pal32[c].p.g;
				r[RGB_B] = np2_pal32[c].p.b;
			}
		}
		s += SURFACE_WIDTH;
		d += layer.width;
	} while(++layer.y < layer.maxy);

	layer.surface = d;
	layer.src1scrn = s;
}


//	text + grph / 24bit
static void np2layer24n_tg(void) {

	BYTE	*d = layer.surface;
	BYTE	*s1 = layer.src1scrn;
	BYTE	*s2 = layer.src2scrn;
	int		x;
	BYTE	*r;
	BYTE	c;

	do {
		if (renewal_line[layer.y] & layer.bit) {
			renewal_line[layer.y] &= ~layer.bit;
			c = (s2[0] >> 4) + NP2PAL_TEXT3;
			r = d;
			r[RGB_R] = np2_pal32[c].p.r;
			r[RGB_G] = np2_pal32[c].p.g;
			r[RGB_B] = np2_pal32[c].p.b;
			for (x=1; x<layer.x; x++) {
				c = s1[x-1] + s2[x] + NP2PAL_GRPH;
				r += RGB24_STEP;
				r[RGB_R] = np2_pal32[c].p.r;
				r[RGB_G] = np2_pal32[c].p.g;
				r[RGB_B] = np2_pal32[c].p.b;
			}
			x = s1[x-1] + NP2PAL_GRPH;
			r += RGB24_STEP;
			r[RGB_R] = np2_pal32[c].p.r;
			r[RGB_G] = np2_pal32[c].p.g;
			r[RGB_B] = np2_pal32[c].p.b;
		}
		s1 += SURFACE_WIDTH;
		s2 += SURFACE_WIDTH;
		d += layer.width;
	} while(++layer.y < layer.maxy);

	layer.surface = d;
	layer.src1scrn = s1;
	layer.src2scrn = s2;
}


//	text + (grph:skipline) / 24bit
static void np2layer24n_t_skip(void) {

	BYTE	*d = layer.surface;
	BYTE	*s = layer.src1scrn;
	int		x;
	BYTE	*r;
	BYTE	c;

	do {
		if (renewal_line[layer.y] & layer.bit) {
			renewal_line[layer.y] &= ~layer.bit;
			c = (s[0] >> 4) + NP2PAL_TEXT3;
			r = d;
			r[RGB_R] = np2_pal32[c].p.r;
			r[RGB_G] = np2_pal32[c].p.g;
			r[RGB_B] = np2_pal32[c].p.b;
			for (x=1; x<layer.x; x++) {
				c = s[x] + NP2PAL_GRPH;
				r += RGB24_STEP;
				r[RGB_R] = np2_pal32[c].p.r;
				r[RGB_G] = np2_pal32[c].p.g;
				r[RGB_B] = np2_pal32[c].p.b;
			}
			r += RGB24_STEP;
			r[RGB_R] = np2_pal32[NP2PAL_GRPH].p.r;
			r[RGB_G] = np2_pal32[NP2PAL_GRPH].p.g;
			r[RGB_B] = np2_pal32[NP2PAL_GRPH].p.b;
		}
		s += SURFACE_WIDTH;
		d += layer.width;
		layer.y++;
		if (renewal_line[layer.y] & layer.bit) {
			renewal_line[layer.y] &= ~layer.bit;
			c = (s[0] >> 4) + NP2PAL_TEXT3;
			r = d;
			r[RGB_R] = np2_pal32[c].p.r;
			r[RGB_G] = np2_pal32[c].p.g;
			r[RGB_B] = np2_pal32[c].p.b;
			for (x=1; x<layer.x; x++) {
				c = (s[x] >> 4) + NP2PAL_TEXT;
				r += RGB24_STEP;
				r[RGB_R] = np2_pal32[c].p.r;
				r[RGB_G] = np2_pal32[c].p.g;
				r[RGB_B] = np2_pal32[c].p.b;
			}
			r += RGB24_STEP;
			r[RGB_R] = np2_pal32[NP2PAL_TEXT].p.r;
			r[RGB_G] = np2_pal32[NP2PAL_TEXT].p.g;
			r[RGB_B] = np2_pal32[NP2PAL_TEXT].p.b;
		}
		s += SURFACE_WIDTH;
		d += layer.width;
	} while(++layer.y < layer.maxy);

	layer.surface = d;
	layer.src1scrn = s;
}


//	grph + skipline / 24bit
static void np2layer24n_g_skip(void) {

	BYTE	*d = layer.surface;
	BYTE	*s = layer.src1scrn;
	int		x;
	BYTE	*r;
	BYTE	c;

	do {
		if (renewal_line[layer.y] & layer.bit) {
			renewal_line[layer.y] &= ~layer.bit;
			r = d;
			r[RGB_R] = 0;
			r[RGB_G] = 0;
			r[RGB_B] = 0;
			for (x=0; x<layer.x; x++) {
				c = s[x] + NP2PAL_GRPH;
				r += RGB24_STEP;
				r[RGB_R] = np2_pal32[c].p.r;
				r[RGB_G] = np2_pal32[c].p.g;
				r[RGB_B] = np2_pal32[c].p.b;
			}
		}
		s += SURFACE_WIDTH;
		d += layer.width;
		layer.y++;
		if (renewal_line[layer.y] & layer.bit) {
			renewal_line[layer.y] &= ~layer.bit;
			r = d;
			r[RGB_R] = 0;
			r[RGB_G] = 0;
			r[RGB_B] = 0;
			for (x=0; x<layer.x; x++) {
				c = NP2PAL_TEXT;
				r += RGB24_STEP;
				r[RGB_R] = np2_pal32[c].p.r;
				r[RGB_G] = np2_pal32[c].p.g;
				r[RGB_B] = np2_pal32[c].p.b;
			}
		}
		s += SURFACE_WIDTH;
		d += layer.width;
	} while(++layer.y < layer.maxy);

	layer.surface = d;
	layer.src1scrn = s;
}


//	text + grph + skipline / 24bit
static void np2layer24n_tg_skip(void) {

	BYTE	*d = layer.surface;
	BYTE	*s1 = layer.src1scrn;
	BYTE	*s2 = layer.src2scrn;
	int		x;
	BYTE	*r;
	BYTE	c;

	do {
		if (renewal_line[layer.y] & layer.bit) {
			renewal_line[layer.y] &= ~layer.bit;
			c = (s2[0] >> 4) + NP2PAL_TEXT3;
			r = d;
			r[RGB_R] = np2_pal32[c].p.r;
			r[RGB_G] = np2_pal32[c].p.g;
			r[RGB_B] = np2_pal32[c].p.b;
			for (x=1; x<layer.x; x++) {
				c = s1[x-1] + s2[x] + NP2PAL_GRPH;
				r += RGB24_STEP;
				r[RGB_R] = np2_pal32[c].p.r;
				r[RGB_G] = np2_pal32[c].p.g;
				r[RGB_B] = np2_pal32[c].p.b;
			}
			c = s1[x-1] + NP2PAL_GRPH;
			r += RGB24_STEP;
			r[RGB_R] = np2_pal32[c].p.r;
			r[RGB_G] = np2_pal32[c].p.g;
			r[RGB_B] = np2_pal32[c].p.b;
		}
		s2 += SURFACE_WIDTH;
		d += layer.width;
		layer.y++;
		if (renewal_line[layer.y] & layer.bit) {
			renewal_line[layer.y] &= ~layer.bit;
			c = (s2[0] >> 4) + NP2PAL_TEXT3;
			r = d;
			r[RGB_R] = np2_pal32[c].p.r;
			r[RGB_G] = np2_pal32[c].p.g;
			r[RGB_B] = np2_pal32[c].p.b;
			for (x=1; x<layer.x; x++) {
				c = (s2[x] >> 4) + NP2PAL_TEXT;
				r += RGB24_STEP;
				r[RGB_R] = np2_pal32[c].p.r;
				r[RGB_G] = np2_pal32[c].p.g;
				r[RGB_B] = np2_pal32[c].p.b;
			}
			c = NP2PAL_TEXT;
			r += RGB24_STEP;
			r[RGB_R] = np2_pal32[c].p.r;
			r[RGB_G] = np2_pal32[c].p.g;
			r[RGB_B] = np2_pal32[c].p.b;
		}
		s1 += (SURFACE_WIDTH * 2);
		s2 += SURFACE_WIDTH;
		d += layer.width;
	} while(++layer.y < layer.maxy);

	layer.surface = d;
	layer.src1scrn = s1;
	layer.src2scrn = s2;
}


// --- with extend skipline

//	grph + extend skipline / 24bit
static void np2layer24n_g_ext(void) {

	BYTE	*d = layer.surface;
	BYTE	*s = layer.src1scrn;
	int		x;
	BYTE	*r;
	BYTE	c;

	do {
		if (renewal_line[layer.y] & layer.bit) {
			renewal_line[layer.y] &= ~layer.bit;
			renewal_line[layer.y + 1] |= 0x80;				// next line draw
			r = d;
			r[RGB_R] = 0;
			r[RGB_G] = 0;
			r[RGB_B] = 0;
			for (x=0; x<layer.x; x++) {
				c = s[x] + NP2PAL_GRPH;
				r += RGB24_STEP;
				r[RGB_R] = np2_pal32[c].p.r;
				r[RGB_G] = np2_pal32[c].p.g;
				r[RGB_B] = np2_pal32[c].p.b;
			}
		}
		d += layer.width;
		layer.y++;
		if (renewal_line[layer.y] & layer.bit) {
			renewal_line[layer.y] &= ~layer.bit;
			r = d;
			r[RGB_R] = 0;
			r[RGB_G] = 0;
			r[RGB_B] = 0;
			for (x=0; x<layer.x; x++) {
				c = s[x] + NP2PAL_SKIP;
				r += RGB24_STEP;
				r[RGB_R] = np2_pal32[c].p.r;
				r[RGB_G] = np2_pal32[c].p.g;
				r[RGB_B] = np2_pal32[c].p.b;
			}
		}
		s += (SURFACE_WIDTH * 2);
		d += layer.width;
	} while(++layer.y < layer.maxy);

	layer.surface = d;
	layer.src1scrn = s;
}

//	text + grph + extend skipline / 24bit
static void np2layer24n_tg_ext(void) {

	BYTE	*d = layer.surface;
	BYTE	*s1 = layer.src1scrn;
	BYTE	*s2 = layer.src2scrn;
	int		x;
	BYTE	*r;
	BYTE	c;

	do {
		if (renewal_line[layer.y] & layer.bit) {
			renewal_line[layer.y] &= ~layer.bit;
			renewal_line[layer.y + 1] |= 0x80;				// next line draw
			c = (s2[0] >> 4) + NP2PAL_TEXT3;
			r = d;
			r[RGB_R] = np2_pal32[c].p.r;
			r[RGB_G] = np2_pal32[c].p.g;
			r[RGB_B] = np2_pal32[c].p.b;
			for (x=1; x<layer.x; x++) {
				c = s1[x-1] + s2[x] + NP2PAL_GRPH;
				r += RGB24_STEP;
				r[RGB_R] = np2_pal32[c].p.r;
				r[RGB_G] = np2_pal32[c].p.g;
				r[RGB_B] = np2_pal32[c].p.b;
			}
			c = s1[x-1] + NP2PAL_GRPH;
			r += RGB24_STEP;
			r[RGB_R] = np2_pal32[c].p.r;
			r[RGB_G] = np2_pal32[c].p.g;
			r[RGB_B] = np2_pal32[c].p.b;
		}
		s2 += SURFACE_WIDTH;
		d += layer.width;
		layer.y++;
		if (renewal_line[layer.y] & layer.bit) {
			renewal_line[layer.y] &= ~layer.bit;
			c = (s2[0] >> 4) + NP2PAL_TEXT3;
			r = d;
			r[RGB_R] = np2_pal32[c].p.r;
			r[RGB_G] = np2_pal32[c].p.g;
			r[RGB_B] = np2_pal32[c].p.b;
			for (x=1; x<layer.x; x++) {
				c = s2[x] >> 4;
				if (!c) {
					c = s1[x-1] + NP2PALS_TXT;
				}
				r += RGB24_STEP;
				r[RGB_R] = np2_pal32[c].p.r;
				r[RGB_G] = np2_pal32[c].p.g;
				r[RGB_B] = np2_pal32[c].p.b;
			}
			c = s1[x-1] + NP2PAL_SKIP;
			r += RGB24_STEP;
			r[RGB_R] = np2_pal32[c].p.r;
			r[RGB_G] = np2_pal32[c].p.g;
			r[RGB_B] = np2_pal32[c].p.b;
		}
		s1 += (SURFACE_WIDTH * 2);
		s2 += SURFACE_WIDTH;
		d += layer.width;
	} while(++layer.y < layer.maxy);

	layer.surface = d;
	layer.src1scrn = s1;
	layer.src2scrn = s2;
}

#endif



// ----------------------------------------------------------------- turn left

static void lay24l(void) {

	BYTE	*d = layer.surface;
	int		x;
	BYTE	*r;
	BYTE	c;

	do {
		if (renewal_line[layer.y] & 0x80) {
			renewal_line[layer.y] &= 0x7f;
			for (r=d, x=layer.x; x--;) {
				c = NP2PAL_TEXT2;
				r[RGB_R] = np2_pal32[c].p.r;
				r[RGB_G] = np2_pal32[c].p.g;
				r[RGB_B] = np2_pal32[c].p.b;
				r += layer.width;
			}
		}
		d += RGB24_STEP;
	} while(++layer.y < layer.maxy);
	layer.surface = d;
}


static void np2layer24l_1(void) {

	BYTE	*d = layer.surface;
	BYTE	*s = layer.src1scrn;
	int		x;
	BYTE	*r;
	BYTE	c;

	do {
		if (renewal_line[layer.y] & layer.bit) {
			renewal_line[layer.y] &= ~layer.bit;
			for (r=d, x=layer.x; x--;) {
				c = s[x] + NP2PAL_GRPH;
				r[RGB_R] = np2_pal32[c].p.r;
				r[RGB_G] = np2_pal32[c].p.g;
				r[RGB_B] = np2_pal32[c].p.b;
				r += layer.width;
			}
		}
		s += SURFACE_WIDTH;
		d += RGB24_STEP;
	} while(++layer.y < layer.maxy);
	layer.surface = d;
	layer.src1scrn = s;
}

static void np2layer24l_tg(void) {

	BYTE	*d = layer.surface;
	BYTE	*s1 = layer.src1scrn;
	BYTE	*s2 = layer.src2scrn;
	int		x;
	BYTE	*r;
	BYTE	c;

	do {
		if (renewal_line[layer.y] & layer.bit) {
			renewal_line[layer.y] &= ~layer.bit;
			for (r=d, x=layer.x; x--;) {
				c = s1[x] + s2[x] + NP2PAL_GRPH;
				r[RGB_R] = np2_pal32[c].p.r;
				r[RGB_G] = np2_pal32[c].p.g;
				r[RGB_B] = np2_pal32[c].p.b;
				r += layer.width;
			}
		}
		s1 += SURFACE_WIDTH;
		s2 += SURFACE_WIDTH;
		d += RGB24_STEP;
	} while(++layer.y < layer.maxy);

	layer.surface = d;
	layer.src1scrn = s1;
	layer.src2scrn = s2;
}

static void np2layer24l_t_skip(void) {

	BYTE	*d = layer.surface;
	BYTE	*s = layer.src1scrn;
	int		x;
	BYTE	*r;
	BYTE	c;

	do {
		if (renewal_line[layer.y] & layer.bit) {
			renewal_line[layer.y] &= ~layer.bit;
			for (r=d, x=layer.x; x--;) {
				c = s[x] + NP2PAL_GRPH;
				r[RGB_R] = np2_pal32[c].p.r;
				r[RGB_G] = np2_pal32[c].p.g;
				r[RGB_B] = np2_pal32[c].p.b;
				r += layer.width;
			}
		}
		s += SURFACE_WIDTH;
		d += RGB24_STEP;
		layer.y++;
		if (renewal_line[layer.y] & layer.bit) {
			renewal_line[layer.y] &= ~layer.bit;
			for (r=d, x=layer.x; x--;) {
				c = (s[x] >> 4) + NP2PAL_TEXT;
				r[RGB_R] = np2_pal32[c].p.r;
				r[RGB_G] = np2_pal32[c].p.g;
				r[RGB_B] = np2_pal32[c].p.b;
				r += layer.width;
			}
		}
		s += SURFACE_WIDTH;
		d += RGB24_STEP;
	} while(++layer.y < layer.maxy);

	layer.surface = d;
	layer.src1scrn = s;
}

static void np2layer24l_g_skip(void) {

	BYTE	*d = layer.surface;
	BYTE	*s = layer.src1scrn;
	int		x;
	BYTE	*r;
	BYTE	c;

	do {
		if (renewal_line[layer.y] & layer.bit) {
			renewal_line[layer.y] &= ~layer.bit;
			for (r=d, x=layer.x; x--;) {
				c = s[x] + NP2PAL_GRPH;
				r[RGB_R] = np2_pal32[c].p.r;
				r[RGB_G] = np2_pal32[c].p.g;
				r[RGB_B] = np2_pal32[c].p.b;
				r += layer.width;
			}
		}
		s += SURFACE_WIDTH;
		d += RGB24_STEP;
		layer.y++;
		if (renewal_line[layer.y] & layer.bit) {
			renewal_line[layer.y] &= ~layer.bit;
			for (r=d, x=layer.x; x--;) {
				r[RGB_R] = np2_pal32[NP2PAL_TEXT].p.r;
				r[RGB_G] = np2_pal32[NP2PAL_TEXT].p.g;
				r[RGB_B] = np2_pal32[NP2PAL_TEXT].p.b;
				r += layer.width;
			}
		}
		s += SURFACE_WIDTH;
		d += RGB24_STEP;
	} while(++layer.y < layer.maxy);

	layer.surface = d;
	layer.src1scrn = s;
}

static void np2layer24l_tg_skip(void) {

	BYTE	*d = layer.surface;
	BYTE	*s1 = layer.src1scrn;
	BYTE	*s2 = layer.src2scrn;
	int		x;
	BYTE	*r;
	BYTE	c;

	do {
		if (renewal_line[layer.y] & layer.bit) {
			renewal_line[layer.y] &= ~layer.bit;
			for (r=d, x=layer.x; x--;) {
				c = s1[x] + s2[x] + NP2PAL_GRPH;
				r[RGB_R] = np2_pal32[c].p.r;
				r[RGB_G] = np2_pal32[c].p.g;
				r[RGB_B] = np2_pal32[c].p.b;
				r += layer.width;
			}
		}
		s2 += SURFACE_WIDTH;
		d += RGB24_STEP;
		layer.y++;
		if (renewal_line[layer.y] & layer.bit) {
			renewal_line[layer.y] &= ~layer.bit;
			for (r=d, x=layer.x; x--;) {
				c = (s2[x] >> 4) + NP2PAL_TEXT;
				r[RGB_R] = np2_pal32[c].p.r;
				r[RGB_G] = np2_pal32[c].p.g;
				r[RGB_B] = np2_pal32[c].p.b;
				r += layer.width;
			}
		}
		s1 += (SURFACE_WIDTH * 2);
		s2 += SURFACE_WIDTH;
		d += RGB24_STEP;
	} while(++layer.y < layer.maxy);

	layer.surface = d;
	layer.src1scrn = s1;
	layer.src2scrn = s2;
}



// ---------------------------------------------------------------- turn right

static void lay24r(void) {

	BYTE	*d;
	int		x;
	BYTE	*r;
	BYTE	c;

	d = layer.surface + ((layer.maxy - layer.y) * RGB24_STEP);
	do {
		d -= RGB24_STEP;
		if (renewal_line[layer.y] & 0x80) {
			renewal_line[layer.y] &= 0x7f;
			for (r=d, x=0; x<layer.x; x++) {
				c = NP2PAL_TEXT2;
				r[RGB_R] = np2_pal32[c].p.r;
				r[RGB_G] = np2_pal32[c].p.g;
				r[RGB_B] = np2_pal32[c].p.b;
				r += layer.width;
			}
		}
	} while(++layer.y < layer.maxy);
}


static void np2layer24r_1(void) {

	BYTE	*d;
	BYTE	*s = layer.src1scrn;
	int		x;
	BYTE	*r;
	BYTE	c;

	d = layer.surface + ((layer.maxy - layer.y) * RGB24_STEP);
	do {
		d -= RGB24_STEP;
		if (renewal_line[layer.y] & layer.bit) {
			renewal_line[layer.y] &= ~layer.bit;
			for (r=d, x=0; x<layer.x; x++) {
				c = s[x] + NP2PAL_GRPH;
				r[RGB_R] = np2_pal32[c].p.r;
				r[RGB_G] = np2_pal32[c].p.g;
				r[RGB_B] = np2_pal32[c].p.b;
				r += layer.width;
			}
		}
		s += SURFACE_WIDTH;
	} while(++layer.y < layer.maxy);

	layer.src1scrn = s;
}

static void np2layer24r_tg(void) {

	BYTE	*d;
	BYTE	*s1 = layer.src1scrn;
	BYTE	*s2 = layer.src2scrn;
	int		x;
	BYTE	*r;
	BYTE	c;

	d = layer.surface + ((layer.maxy - layer.y) * RGB24_STEP);
	do {
		d -= RGB24_STEP;
		if (renewal_line[layer.y] & layer.bit) {
			renewal_line[layer.y] &= ~layer.bit;
			for (r=d, x=0; x<layer.x; x++) {
				c = s1[x] + s2[x] + NP2PAL_GRPH;
				r[RGB_R] = np2_pal32[c].p.r;
				r[RGB_G] = np2_pal32[c].p.g;
				r[RGB_B] = np2_pal32[c].p.b;
				r += layer.width;
			}
		}
		s1 += SURFACE_WIDTH;
		s2 += SURFACE_WIDTH;
	} while(++layer.y < layer.maxy);

	layer.src1scrn = s1;
	layer.src2scrn = s2;
}

static void np2layer24r_t_skip(void) {

	BYTE	*d;
	BYTE	*s = layer.src1scrn;
	int		x;
	BYTE	*r;
	BYTE	c;

	d = layer.surface + ((layer.maxy - layer.y) * RGB24_STEP);
	do {
		d -= RGB24_STEP;
		if (renewal_line[layer.y] & layer.bit) {
			renewal_line[layer.y] &= ~layer.bit;
			for (r=d, x=0; x<layer.x; x++) {
				c = s[x] + NP2PAL_GRPH;
				r[RGB_R] = np2_pal32[c].p.r;
				r[RGB_G] = np2_pal32[c].p.g;
				r[RGB_B] = np2_pal32[c].p.b;
				r += layer.width;
			}
		}
		s += SURFACE_WIDTH;
		layer.y++;
		d -= RGB24_STEP;
		if (renewal_line[layer.y] & layer.bit) {
			renewal_line[layer.y] &= ~layer.bit;
			for (r=d, x=0; x<layer.x; x++) {
				c = (s[x] >> 4) + NP2PAL_TEXT;
				r[RGB_R] = np2_pal32[c].p.r;
				r[RGB_G] = np2_pal32[c].p.g;
				r[RGB_B] = np2_pal32[c].p.b;
				r += layer.width;
			}
		}
		s += SURFACE_WIDTH;
	} while(++layer.y < layer.maxy);

	layer.src1scrn = s;
}

static void np2layer24r_g_skip(void) {

	BYTE	*d;
	BYTE	*s = layer.src1scrn;
	int		x;
	BYTE	*r;
	BYTE	c;

	d = layer.surface + ((layer.maxy - layer.y) * RGB24_STEP);
	do {
		d -= RGB24_STEP;
		if (renewal_line[layer.y] & layer.bit) {
			renewal_line[layer.y] &= ~layer.bit;
			for (r=d, x=0; x<layer.x; x++) {
				c = s[x] + NP2PAL_GRPH;
				r[RGB_R] = np2_pal32[c].p.r;
				r[RGB_G] = np2_pal32[c].p.g;
				r[RGB_B] = np2_pal32[c].p.b;
				r += layer.width;
			}
		}
		s += SURFACE_WIDTH;
		layer.y++;
		d -= RGB24_STEP;
		if (renewal_line[layer.y] & layer.bit) {
			renewal_line[layer.y] &= ~layer.bit;
			for (r=d, x=0; x<layer.x; x++) {
				r[RGB_R] = np2_pal32[NP2PAL_TEXT].p.r;
				r[RGB_G] = np2_pal32[NP2PAL_TEXT].p.g;
				r[RGB_B] = np2_pal32[NP2PAL_TEXT].p.b;
				r += layer.width;
			}
		}
		s += SURFACE_WIDTH;
	} while(++layer.y < layer.maxy);

	layer.src1scrn = s;
}

static void np2layer24r_tg_skip(void) {

	BYTE	*d;
	BYTE	*s1 = layer.src1scrn;
	BYTE	*s2 = layer.src2scrn;
	int		x;
	BYTE	*r;
	BYTE	c;

	d = layer.surface + ((layer.maxy - layer.y) * RGB24_STEP);
	do {
		d -= RGB24_STEP;
		if (renewal_line[layer.y] & layer.bit) {
			renewal_line[layer.y] &= ~layer.bit;
			for (r=d, x=0; x<layer.x; x++) {
				c = s1[x] + s2[x] + NP2PAL_GRPH;
				r[RGB_R] = np2_pal32[c].p.r;
				r[RGB_G] = np2_pal32[c].p.g;
				r[RGB_B] = np2_pal32[c].p.b;
				r += layer.width;
			}
		}
		s2 += SURFACE_WIDTH;
		layer.y++;
		d -= RGB24_STEP;
		if (renewal_line[layer.y] & layer.bit) {
			renewal_line[layer.y] &= ~layer.bit;
			for (r=d, x=0; x<layer.x; x++) {
				c = (s2[x] >> 4) + NP2PAL_TEXT;
				r[RGB_R] = np2_pal32[c].p.r;
				r[RGB_G] = np2_pal32[c].p.g;
				r[RGB_B] = np2_pal32[c].p.b;
				r += layer.width;
			}
		}
		s1 += (SURFACE_WIDTH * 2);
		s2 += SURFACE_WIDTH;
	} while(++layer.y < layer.maxy);

	layer.src1scrn = s1;
	layer.src2scrn = s2;
}




void (*layer24tbl[])(void) = {

		// Ρޥ
		lay24,	np2layer24_1,		np2layer24_1,		np2layer24_tg,
		lay24,	np2layer24_t_skip,	np2layer24_g_skip,	np2layer24_tg_skip,
		lay24,	np2layer24_t_skip,	np2layer24_g_ext,	np2layer24_tg_ext,

		lay24n,	np2layer24n_t,		np2layer24n_g,		np2layer24n_tg,
		lay24n,	np2layer24n_t_skip,	np2layer24n_g_skip,	np2layer24n_tg_skip,
		lay24n,	np2layer24n_t_skip,	np2layer24n_g_ext,	np2layer24n_tg_ext,


		// 
		lay24l,	np2layer24l_1,		np2layer24l_1,		np2layer24l_tg,
		lay24l,	np2layer24l_t_skip,	np2layer24l_g_skip,	np2layer24l_tg_skip,
		lay24l,	np2layer24l_t_skip,	np2layer24l_g_skip,	np2layer24l_tg_skip,

		lay24l,	np2layer24l_1,		np2layer24l_1,		np2layer24l_tg,
		lay24l,	np2layer24l_t_skip,	np2layer24l_g_skip,	np2layer24l_tg_skip,
		lay24l,	np2layer24l_t_skip,	np2layer24l_g_skip,	np2layer24l_tg_skip,


		// 
		lay24r,	np2layer24r_1,		np2layer24r_1,		np2layer24r_tg,
		lay24r,	np2layer24r_t_skip,	np2layer24r_g_skip,	np2layer24r_tg_skip,
		lay24r,	np2layer24r_t_skip,	np2layer24r_g_skip,	np2layer24r_tg_skip,

		lay24r,	np2layer24r_1,		np2layer24r_1,		np2layer24r_tg,
		lay24r,	np2layer24r_t_skip,	np2layer24r_g_skip,	np2layer24r_tg_skip,
		lay24r,	np2layer24r_t_skip,	np2layer24r_g_skip,	np2layer24r_tg_skip,
};
