#include	"compiler.h"

#include	"np2.h"
#include	"iobridge.h"
#include	"pc_core.h"
#include	"sysport.h"
#include	"beep.h"
#include	"itimer.h"
#include	"nevent.h"
#include	"np2sound.h"

	BEEP_T		beep;
	BEEP_EVENT	beepevent;


void beep_reset(void) {										// ver0.27a

	ZeroMemory(&beep, sizeof(beep));
	beep.mode = 1;
}

void beep_hzset(WORD cnt) {

	double	hz;

	sound_make();
	beep.hz = 0;
	if (cnt) {
		hz = 65536.0 / 4.0 * pc.baseclock / np2sound.rate / cnt;
		if (hz < 0x10000) {
			beep.hz = (WORD)hz;
		}
	}
}

// --------------------------------------------------------------------------

void beep_modeset(void) {

	BYTE	newmode = (itimer.mode[1] >> 2) & 3;

	if (beep.mode != newmode) {
		sound_make();
		beep.mode = newmode;
		if (!newmode) {					// mode:#0, #1
			beep_eventinit();
		}
	}
}

static void beep_eventset(void) {

	int enable = beepevent.beep_low & beep.enable;
	if (beepevent.enable != enable) {
		beepevent.enable = enable;
		if (beepevent.events < BEEPEVENT_MAX) {
			int	 clock;
			clock = nevent.clock + nevent.baseclock - nevent.remainclock;
			beepevent.event[beepevent.events].clock = clock - beepevent.clock;
			beepevent.clock = clock;
			beepevent.event[beepevent.events++].enable = enable;
		}
	}
}

void beep_eventinit(void) {

	// beepevent.event[] ʳ򥼥ꥢƤޤ
	ZeroMemory(&beepevent, (int)(&(((BEEP_EVENT *)NULL)->event)));
	beepevent.clock = np2sound.lastclock;
				//nevent.clock + nevent.baseclock - nevent.remainclock;
}

void beep_eventreset(void) {

	beepevent.lastenable = beepevent.enable;
	beepevent.clock = np2sound.lastclock;
	beepevent.events = 0;
}


void beep_lheventset(int beep_low) {

	if (beepevent.beep_low != beep_low) {
		sound_make();
		beepevent.beep_low = beep_low;
		if (!beep.mode) {
			beep_eventset();
		}
	}
}

void beep_oneventset(void) {

	int		enable;

	enable = ((systemport.c & 8)?0:1);

	if (beep.enable != enable) {
		sound_make();
		beep.enable = enable;
		if (!beep.mode) {
			beep_eventset();
		}
	}
}


// --------------------------------------------------------------------------
#ifdef SOUND_BEEP_ASM

LABEL void __fastcall beep_update(void *buf, int buflength) {

		__asm {
				cmp		edx, 0
				je		beep_end

				cmp		beep.mode, 1
				jc		beep_onshot
				jne		beep_end

				cmp		beep.enable, 0
				je		beep_end

				pushad
				mov		si, beep.hz
				or		si, si
				je		beep_mkend
				movzx	edi, np2cfg.BEEP_VOL
				or		edi, edi
				je		beep_mkend
				shl		edi, 8

beep_lp1:		mov		al, 4
beep_lp2:		add		beep.cnt, si
				js		short beep_mina
					add		[ecx], edi
					add		[ecx+4], edi
				jmp		short beep_lped
beep_mina:			sub		[ecx], edi
					sub		[ecx+4], edi
beep_lped:		dec		al
				jne		beep_lp2
				add		ecx, 8
				dec		edx
				jne		beep_lp1

beep_mkend:		popad
beep_end:		ret

beep_onshot:	cmp		beepevent.events, 0
				je		beep_end
				cmp		pc.sampleclock, 0
				je		beep_end

				pushad
				mov		esi, offset beepevent.event
				movzx	edi, np2cfg.BEEP_VOL
				or		edi, edi
				je		beep1s_mkend
				shl		edi, 11

beep1shot_lp:	mov		ebp, pc.sampleclock
				xor		eax, eax

beep1shot_lp1:	mov		ebx, [esi]BEEP1EVENT.clock
				cmp		ebp, ebx
				jc		beep1s_envof

				cmp		beepevent.lastenable, 0
				je		beep1s_rmnchk
				add		eax, ebx
beep1s_rmnchk:	mov		edx, [esi]BEEP1EVENT.enable
				mov		beepevent.lastenable, edx
				add		esi, type BEEP1EVENT
				dec		beepevent.events
				je		beep_nonevents
				sub		ebp, ebx
				jne		beep1shot_lp1
				jmp		short beep1s_calcvol

beep1s_envof:	sub		[esi]BEEP1EVENT.clock, ebp
				cmp		beepevent.lastenable, 0
				je		beep1s_calcvol
				add		eax, ebp
beep1s_calcvol:	mul		edi
				div		pc.sampleclock
				add		[ecx], eax
				add		[ecx+4], eax
				add		ecx, 8
				dec		dword ptr [esp + 20]			; pushadedx
				jne		beep1shot_lp
				jmp		beep1s_mkend

beep_nonevents:	cmp		beepevent.lastenable, 0
				je		beep1s_calc2
				sub		ebp, ebx
;				shr		ebp, 1
				add		eax, ebp
beep1s_calc2:	mul		edi
				div		pc.sampleclock
				add		[ecx], eax
				add		[ecx+4], eax
				add		ecx, 8
				dec		dword ptr [esp + 20]			; pushadedx
				je		beep1s_mkend
				cmp		beepevent.lastenable, 0
				je		beep1s_mkend
beep1shot_lp2:	add		[ecx], edi
				add		[ecx+4], edi
				add		ecx, 8
				dec		dword ptr [esp + 20]			; pushadedx
				jne		beep1shot_lp2

beep1s_mkend:
				popad
				ret
		}
}

#else

INLINE void beepupdate_oneshot(long *bufp, int buflength) {

	long		vol;
	long		smp;
	DWORD		remain;
	BEEP1EVENT	*bev;

	vol = (long)np2cfg.BEEP_VOL << 11;
	bev = beepevent.event;

	do {
		remain = pc.sampleclock;
		smp = 0;
		while(remain >= bev->clock) {
			if (--beepevent.events) {
				remain -= bev->clock;
				if (beepevent.lastenable) {
					smp += bev->clock;
				}
				beepevent.lastenable = bev->enable;
				bev++;
			}
			else {								// ץ뤬ݤʤʤ
				if (beepevent.lastenable) {
					smp += remain;
					smp *= vol;
					smp /= pc.sampleclock;
					do {
						bufp[0] += smp;
						bufp[1] += smp;
						bufp += 2;
						smp >>= 1;
					} while(--buflength);
				}
				else {
					smp *= vol;
					smp /= pc.sampleclock;
					bufp[0] += smp;
					bufp[1] += smp;
				}
				return;
			}
		}
		bev->clock -= remain;
		if (beepevent.lastenable) {
			smp += remain;
		}
		smp *= vol;
		smp /= pc.sampleclock;
		bufp[0] += smp;
		bufp[1] += smp;
		bufp += 2;
	} while(--buflength);
}

INLINE void beepupdate_rategenerate(long *bufp, int buflength) {

	long	vol;
	long	smp;
	int		cnt;

	vol = (long)np2cfg.BEEP_VOL << 8;
	do {
		smp = 0;
		cnt = 4;
		do {
			beep.cnt += beep.hz;
			smp += vol * ((beep.cnt & 0x8000)?1:-1);
		} while(--cnt);
		bufp[0] += smp;
		bufp[1] += smp;
		bufp += 2;
	} while(--buflength);
}


void SOUND_CALL beep_update(void *buf, int buflength) {

	if ((buflength) && (np2cfg.BEEP_VOL)) {
		switch(beep.mode) {
			case 0:									// one shot
				if ((beepevent.events) && (pc.sampleclock)) {
					beepupdate_oneshot((long *)buf, buflength);
				}
				break;
			case 1:									// rate generator
				if ((beep.enable) && (beep.hz)) {
					beepupdate_rategenerate((long *)buf, buflength);
				}
				break;
		}
	}
}

#endif

