/*	$Id: ini.c,v 1.13 2002/06/25 15:28:44 nonaka Exp $	*/

#include "compiler.h"

#include <sys/types.h>
#include <sys/stat.h>
#include <ctype.h>

#include "np2.h"
#include "iobridge.h"
#include "pc_core.h"

#include "audio.h"
#include "bios.h"
#include "dosio.h"
#include "ini.h"
#include "midi.h"
#include "milstr.h"
#include "xkeyboard.h"

#define	CFGLEN	(MAX_PATH + 32)

static int force_init = 0;

BYTE init_flag = 0;
char ini_title[] = "NekoProjectII";

char config[MAX_PATH] = "";

char fddfolder[MAX_PATH] = "";
char hddfolder[MAX_PATH] = "";
char bmpfilefolder[MAX_PATH] = "";
char mimpideffile[MAX_PATH] = "";
char fontfile[MAX_PATH] = "";

typedef struct {
	char	item[9];
	BYTE	itemtype;
	WORD	flag;
	void	*value;
	DWORD	arg;
} INI_ITEM;

#define	INI_RO		0x0001
#define	INIAND		0x0002
#define	INIMAX		0x0004
#define	INIA4		0x0008
#define	INIEND		0x8000

#define	INI_STR		0x00
#define	INI_BOOL	0x01
#define	INI_ARG		0x02
#define	INI_BYTE	0x11
#define	INI_WORD	0x12
#define	INI_DWORD	0x14
#define	INI_HEXB	0x21
#define	INI_HEXD	0x24
#define	INI_EX		0x80

static INI_ITEM iniitem[] =
{
{"np2title", INI_STR,	INI_RO,		np2cfg.titles,	sizeof(np2cfg.titles)},
{"np2winid", INI_EX|0,	INI_RO,		np2cfg.winid,		0},
{"WindposX", INI_DWORD,	0,		&winx,			0},
{"WindposY", INI_DWORD,	0,		&winy,			0},
{"Win_Snap", INI_BOOL,	0,		&np2cfg.WINSNAP,	0},
{"padsizex", INI_DWORD,	INI_RO|INIMAX,	&np2cfg.paddingx,	32},
{"padsizey", INI_DWORD,	INI_RO|INIMAX,	&np2cfg.paddingy,	32},
{"paddingx", INI_DWORD,	INI_RO|INIMAX,	&np2cfg.paddingx,	32},
{"paddingy", INI_DWORD,	INI_RO|INIMAX,	&np2cfg.paddingy,	32},

{"FDfolder", INI_STR,	0,		fddfolder,		MAX_PATH},
{"HDfolder", INI_STR,	0,		hddfolder,		MAX_PATH},
{"bmap_Dir", INI_STR,	0,		bmpfilefolder,		MAX_PATH},
{"fontfile", INI_STR,	0,		fontfile,		MAX_PATH},

{"clk_base", INI_DWORD,	0,		&np2cfg.baseclock,	0},
{"clk_mult", INI_DWORD,	0,		&np2cfg.multiple,	0},

{"DIPswtch", INI_ARG,	0,		np2cfg.dipsw,		3},
{"MEMswtch", INI_ARG,	0,		np2cfg.memsw,		8},
{"ExMemory", INI_BYTE,	INIMAX,		&np2cfg.EXTMEM,		13},
{"ITF_WORK", INI_BOOL,	INI_RO,		&np2cfg.ITF_WORK,	0},

{"HDD1FILE", INI_STR,	0,		hd_name[0],		MAX_PATH},
{"HDD2FILE", INI_STR,	0,		hd_name[1],		MAX_PATH},
{"Removabl", INI_EX|1,	0,		&np2cfg.REMOVEHD,	0},

{"SampleHz", INI_WORD,	0,		&np2cfg.samplingrate,	0},
{"Latencys", INI_WORD,	0,		&np2cfg.delayms,	0},
{"SNDboard", INI_HEXB,	INIAND,		&np2cfg.SOUND_SW,	0x7f},
{"BEEP_vol", INI_BYTE,	INIAND,		&np2cfg.BEEP_VOL,	0x03},
{"SND14vol", INI_ARG,	INIA4,		np2cfg.vol14,		5},
{"opt26BRD", INI_HEXB,	0,		&np2cfg.snd26opt,	0},
{"opt86BRD", INI_HEXB,	0,		&np2cfg.snd86opt,	0},
{"optSPBRD", INI_HEXB,	0,		&np2cfg.spbopt,		0},
{"optMPU98", INI_HEXB,	0,		&np2cfg.mpuopt,		0},

{"volume_F", INI_BYTE,	INIMAX,		&np2cfg.vol_fm,		128},
{"volume_S", INI_BYTE,	INIMAX,		&np2cfg.vol_ssg,	128},
{"volume_A", INI_BYTE,	INIMAX,		&np2cfg.vol_adpcm,	128},
{"volume_P", INI_BYTE,	INIMAX,		&np2cfg.vol_pcm,	128},
{"volume_R", INI_BYTE,	INIMAX,		&np2cfg.vol_rhythm,	128},

{"MIDI_mod", INI_EX|2,	0,		&np2cfg.MIDIMODULE,	0},
{"MIDI_rst", INI_BOOL,	0,		&np2cfg.MIDIRESET,	0},
{"MIMPIdef", INI_STR,	0,		mimpideffile,		MAX_PATH},

{"Seek_Snd", INI_BOOL,	0,		&np2cfg.MOTOR,		0},
{"Seek_Vol", INI_BYTE,	INIMAX,		&np2cfg.MOTORVOL,	100},

{"keyboard", INI_EX|3,	INI_RO,		&np2cfg.KEYBOARD,	0},
{"F12_COPY", INI_BYTE,	0,		&np2cfg.F12COPY,	0},
{"Joystick", INI_BOOL,	0,		&np2cfg.JOYPAD1,	0},
{"Joy1_btn", INI_ARG,	0,		np2cfg.JOY1BTN,		4},
{"btnRAPID", INI_BOOL,	0,		&np2cfg.BTN_RAPID,	0},
{"btn_MODE", INI_BOOL,	0,		&np2cfg.BTN_MODE,	0},
{"Mouse_sw", INI_BOOL,	0,		&np2cfg.MOUSE_SW,	0},
{"MS_RAPID", INI_BOOL,	0,		&np2cfg.MOUSERAPID,	0},

{"com_port", INI_BYTE,	INIMAX,		&np2cfg.RS_COM,		4},
{"rs_speed", INI_DWORD,	0,		&np2cfg.RS_SPEED,	0},

{"backgrnd", INI_BYTE,	INIAND,		&np2cfg.background,	0x03},
{"VRAMwait", INI_ARG,	0,		np2cfg.wait,		6},
{"DspClock", INI_BYTE,	INIAND,		&np2cfg.DISPCLK,	0x03},
{"DispSync", INI_BOOL,	0,		&np2cfg.DISPSYNC,	0},
{"Real_Pal", INI_BOOL,	0,		&np2cfg.RASTER,		0},
{"s_NOWAIT", INI_BOOL,	0,		&np2cfg.NOWAIT,		0},
{"SkpFrame", INI_BYTE,	0,		&np2cfg.DRAW_SKIP,	0},
{"uPD72020", INI_BOOL,	0,		&np2cfg.uPD72020,	0},
{"GRCG_EGC", INI_BYTE,	INIAND,		&np2cfg.grcg,		0x03},
{"color16b", INI_BOOL,	0,		&np2cfg.color16,	0},
{"skipline", INI_BOOL,	0,		&np2cfg.skipline,	0},
{"skplight", INI_WORD,	0,		&np2cfg.skiplight,	0},
{"LCD_MODE", INI_BYTE,	INIAND,		&np2cfg.LCD_MODE,	0x03},
{"BG_COLOR", INI_HEXD,	INI_RO|INIAND,	&np2cfg.BG_COLOR,	0xffffff},
{"FG_COLOR", INI_HEXD,	INI_RO|INIAND,	&np2cfg.FG_COLOR,	0xffffff},

{"clocknow", INI_BYTE,	0,		&np2cfg.clockx,		0},
{"clockfnt", INI_BYTE,	0,		&np2cfg.clockfnt,	0},
{"clock_up", INI_HEXD,	INI_RO|INIAND,	&np2cfg.clockcolor1,	0xffffff},
{"clock_dn", INI_HEXD,	INI_RO|INIAND,	&np2cfg.clockcolor2,	0xffffff},

{"I286SAVE", INI_BOOL,	INI_RO,		&np2cfg.I286SAVE,	0},

/* Xnp2 */
{"MseRatio", INI_HEXB,	0,		&np2cfg.mouse_move_ratio, 0},

{"Midi_Dev", INI_STR,	0,		MIDIDEV,		MAX_PATH},
{"MidiWait", INI_DWORD,	0,		&MIDI_WAITTIME,		0},

{"SoundDrv", INI_EX|4,	0,		&np2cfg.snddrv,		SNDDRV_DRVMAX},
{"SoundDev", INI_STR,	0,		audiodev,		MAX_PATH},
/* Xnp2 */

{"EndItem",  INI_BOOL,	INIEND,		NULL,			0}
};

static long solveHEX(char *str);
static char *makeBOOL(BYTE value);

static long
solveHEX(char *str)
{

	return milstr_solveHEX(str);
}

static char *
makeBOOL(BYTE value)
{

	return value ? "true" : "false";
}

BYTE
solveBOOL(LPSTR str)
{

	if ((strncasecmp(str, "TRUE", 4) == 0)
	    || (strncasecmp(str, "ON", 2) == 0)
	    || (strncasecmp(str, "+", 1) == 0)
	    || (strncasecmp(str, "1", 1) == 0)
	    || (strncasecmp(str, "ENABLE", 6) == 0))
		return 1;
	return 0;
}

int
set_modulepath(char *path, size_t len)
{
	struct stat sb;

	snprintf(path, len, "%s/%s", getenv("HOME"), ".np2");
	if (stat(path, &sb) < 0) {
		if (mkdir(path, 0700) < 0) {
			perror(path);
			return 1;
		}
	} else {
		if ((sb.st_mode & S_IFDIR) == 0) {
			fprintf(stderr, "%s isn't directory.\n", path);
			return 1;
		}
	}

	snprintf(config, sizeof(config), "%s/%s", modulefile, "config");
	if (stat(config, &sb) >= 0) {
		if (sb.st_mode & S_IFDIR) {
			fprintf(stderr, "%s is directory.\n", config);
			return 1;
		}
	}

	snprintf(fontfile, sizeof(fontfile), "%s/%s", modulefile, "font.bmp");
	if (stat(fontfile, &sb) >= 0) {
		if (sb.st_mode & S_IFDIR) {
			fprintf(stderr, "%s is directory.\n", fontfile);
			return 1;
		}
	}

	return 0;
}

void
initload(void)
{
	char cfgstr[CFGLEN];
	char *arg[8];
	struct stat st;
	INI_ITEM *p;
	DWORD args;
	int i;

	init_flag = 0;

	if (stat(config, &st) < 0) {
		force_init = 1;
		return;
	}

	for (p = iniitem; (p->flag & INIEND) == 0; p++) {
		if ((p->itemtype & 0xf0) == 0) {
			switch (p->itemtype) {
			case INI_STR:
				GetPrivateProfileString(ini_title, p->item,
				    (char *)p->value, (char *)p->value,
				    p->arg, config);
				break;
			case INI_BOOL:
				GetPrivateProfileString(ini_title, p->item,
				    makeBOOL(*((BYTE *)p->value) & 1),
				    cfgstr, CFGLEN, config);
				*(BYTE *)(p->value) = solveBOOL(cfgstr);
				break;
			case INI_ARG:
				GetPrivateProfileString(ini_title, p->item, "",
				    cfgstr, CFGLEN, config);
				args = milstr_getarg(cfgstr, arg, p->arg);
				if (args == p->arg) {
					for (i=0; i<(int)args; i++) {
						((BYTE *)p->value)[i] =
						  (BYTE)solveHEX(arg[i]);
						if (p->flag & INIA4)
						  ((BYTE *)p->value)[i] &= 0x0f;
					}
				}
				break;
			}
		} else if ((p->itemtype & INI_EX) == 0) {
			switch (p->itemtype) {
			case INI_BYTE:
				*(BYTE *)p->value =
				    (BYTE)GetPrivateProfileInt(ini_title,
				        p->item, *(BYTE *)p->value, config);
				break;
			case INI_WORD:
				*(WORD *)p->value =
				    (WORD)GetPrivateProfileInt(ini_title,
				        p->item, *(WORD *)p->value, config);
				break;
			case INI_DWORD:
				*(DWORD *)p->value =
				    (DWORD)GetPrivateProfileInt(ini_title,
				        p->item, *(DWORD *)p->value, config);
				break;
			case INI_HEXB:
				snprintf(cfgstr, sizeof(cfgstr), "%x",
				    *(BYTE *)p->value),
				GetPrivateProfileString(ini_title, p->item,
				    cfgstr, cfgstr, CFGLEN, config);
				*(BYTE *)p->value = (BYTE)solveHEX(cfgstr);
				break;
			case INI_HEXD:
				snprintf(cfgstr, sizeof(cfgstr), "%x",
				    *(DWORD *)p->value),
				GetPrivateProfileString(ini_title, p->item,
				    cfgstr, cfgstr, CFGLEN, config);
				*(DWORD *)p->value = solveHEX(cfgstr);
				break;
			}
			if (p->flag & INIMAX) {
				switch ((p->itemtype) & 0x0f) {
				case 1:
					if (*(BYTE *)p->value > (BYTE)p->arg)
					    *(BYTE *)p->value = (BYTE)p->arg;
					break;
				case 2:
					if (*(WORD *)p->value > (WORD)p->arg)
					    *(WORD *)p->value = (WORD)p->arg;
					break;
				case 4:
					if (*(DWORD *)p->value > (DWORD)p->arg)
					    *(DWORD *)p->value = (DWORD)p->arg;
					break;
				}
			}
			if (p->flag & INIAND) {
				switch((p->itemtype) & 0x0f) {
				case 1:
					*(BYTE *)p->value &= (BYTE)p->arg;
					break;
				case 2:
					*(WORD *)p->value &= (WORD)p->arg;
					break;
				case 4:
					*(DWORD *)p->value &= (DWORD)p->arg;
					break;
				}
			}
		} else {
			switch (p->itemtype) {
			case INI_EX|0:				// winid
				GetPrivateProfileString(ini_title, p->item,
				    (char *)p->value, cfgstr, CFGLEN, config);
				for (i = 0; i < 3; i++)
					if (isalnum(cfgstr[i]))
					    ((BYTE *)p->value)[i] = cfgstr[i];
				break;

			case INI_EX|1:				// remove hdd
				*(BYTE *)p->value =
				    (BYTE)((GetPrivateProfileInt(ini_title,
				      p->item, ((*(BYTE *)p->value) & 1) + 1,
				      config) - 1) & 1);
				break;

			case INI_EX|2:				// midi module
				GetPrivateProfileString(ini_title, p->item,
				    MIDI_num2module(*(BYTE *)p->value),
				    cfgstr, CFGLEN, config);
				*(BYTE *)p->value = MIDI_module2num(cfgstr);
				break;

			case INI_EX|3:				// keyboard
				GetPrivateProfileString(ini_title, p->item, "",
				    cfgstr, CFGLEN, config);
				if ((strcasecmp(cfgstr, "DOS") == 0)
				    || (strcasecmp(cfgstr, "106") == 0)
				    || (strcasecmp(cfgstr, "JP") == 0)
				    || (strcasecmp(cfgstr, "PCAT") == 0)
				    || (strcasecmp(cfgstr, "AT") == 0)) {
					*(BYTE *)p->value = KEYBOARD_JP106;
				} else if ((strcasecmp(cfgstr, "PC98") == 0)
				    || (strcasecmp(cfgstr, "98") == 0)) {
					*(BYTE *)p->value = KEYBOARD_PC98;
				} else if ((strcasecmp(cfgstr, "ASCII") == 0)
				    || (strcasecmp(cfgstr, "101") == 0)
				    || (strcasecmp(cfgstr, "US") == 0)
				    || (strcasecmp(cfgstr, "EN") == 0)) {
					*(BYTE *)p->value = KEYBOARD_US101;
				}
				break;

			case INI_EX|4:				// Sound driver
				GetPrivateProfileString(ini_title, p->item,
				    "", cfgstr, CFGLEN, config);
				*(BYTE *)p->value = audio_drv2num(cfgstr);
				break;
			}
		}
	}
}

void
initsave(void)
{
	char cfgstr[CFGLEN];
	INI_ITEM *p;
	int i;

	if (force_init == 0 && init_flag == 0)
		return;

	for (p = iniitem; (p->flag & INIEND) == 0; p++) {
		if (p->flag & INI_RO) {
			/* Nothing to do. */
		} else if (!(p->itemtype & INI_EX)) {
			switch (p->itemtype) {
			case INI_STR:
				WritePrivateProfileString(ini_title, p->item,
				    (char *)p->value, config);
				break;
			case INI_BOOL:
				WritePrivateProfileString(ini_title, p->item,
				  makeBOOL(*((BYTE *)p->value) & 1), config);
				break;
			case INI_ARG:
				for (i = 0; i < (int)p->arg; i++)
					sprintf(&cfgstr[i*3], "%02x ",
					    ((BYTE *)p->value)[i]);
				cfgstr[i*3-1] = '\0';
				WritePrivateProfileString(ini_title, p->item,
				    cfgstr, config);
				break;
			case INI_BYTE:
				snprintf(cfgstr, sizeof(cfgstr), "%u",
				    *(BYTE *)p->value);
				WritePrivateProfileString(ini_title, p->item,
				    cfgstr, config);
				break;
			case INI_WORD:
				snprintf(cfgstr, sizeof(cfgstr), "%u",
				    *(WORD *)p->value);
				WritePrivateProfileString(ini_title, p->item,
				    cfgstr, config);
				break;
			case INI_DWORD:
				snprintf(cfgstr, sizeof(cfgstr), "%u",
				    *(DWORD *)p->value);
				WritePrivateProfileString(ini_title, p->item,
				    cfgstr, config);
				break;
			case INI_HEXB:
				snprintf(cfgstr, sizeof(cfgstr), "%x",
				    *(BYTE *)p->value);
				WritePrivateProfileString(ini_title, p->item,
				    cfgstr, config);
				break;
			case INI_HEXD:
				snprintf(cfgstr, sizeof(cfgstr), "%x",
				    *(DWORD *)p->value);
				WritePrivateProfileString(ini_title, p->item,
				    cfgstr, config);
				break;
			}
		} else {
			switch(p->itemtype) {
			case INI_EX|1:				// remove hdd
				snprintf(cfgstr, sizeof(cfgstr), "%u",
				    (*(BYTE *)p->value) + 1);
				WritePrivateProfileString(ini_title, p->item,
				    cfgstr, config);
				break;

			case INI_EX|2:				// midi module
				WritePrivateProfileString(ini_title, p->item,
				  MIDI_num2module(*(BYTE *)p->value), config);
				break;

			case INI_EX|4:				// Sound driver
				WritePrivateProfileString(ini_title, p->item,
				  audio_num2drv(*(BYTE *)p->value), config);
				break;
			}
		}
	}
}
