/*	$Id: netbsd.c,v 1.2 2002/06/02 15:04:17 nonaka Exp $	*/

/*
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. All advertising materials mentioning features or use of this software
 *    must display the following acknowledgment:
 *      This product includes software developed by NONAKA Kimihiro.
 * 4. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#if defined(USE_NETBSDAUDIO)

#include "compiler.h"

#include <sys/ioctl.h>
#include <sys/socket.h>
#include <sys/types.h>
#include <unistd.h>
#include <fcntl.h>

#include <sys/audioio.h>

#ifndef	AUDIODEV
#define	AUDIODEV	"/dev/audio"
#endif

#include "audio.h"

static BOOL netbsdaudio_init(DWORD, DWORD);
static BOOL netbsdaudio_term(void);

int
netbsdaudio_setup(void)
{

	audiodrv.init = netbsdaudio_init;
	audiodrv.term = netbsdaudio_term;
	audiodrv.play = audio_generic_play;
	audiodrv.stop = audio_generic_stop;

	return SUCCESS;
}

static BOOL
netbsdaudio_init(DWORD rate, DWORD samples)
{
	audio_info_t a_info;
	u_int blocksize;
	u_int oldhiwat, newhiwat;

	UNUSED(samples);

	if (audio_fd >= 0)
		return SUCCESS;

	audio_fd = open(audiodev[0] == '\0' ? AUDIODEV : audiodev, O_WRONLY);
	if (audio_fd < 0) {
		perror("audio_init: open");
		return FAILURE;
	}

	ioctl(audio_fd, AUDIO_FLUSH, 0);

	AUDIO_INITINFO(&a_info);
	if (ioctl(audio_fd, AUDIO_GETINFO, &a_info) < 0)
		return FAILURE;
	blocksize = a_info.blocksize;
	oldhiwat = a_info.hiwat;

	AUDIO_INITINFO(&a_info);
	a_info.play.encoding = AUDIO_ENCODING_SLINEAR;
	a_info.play.channels = 2;
	a_info.play.precision = 16;
	a_info.play.sample_rate = rate;

	newhiwat = (rate * 4 * 450) / 1000 / blocksize;
	if (newhiwat < oldhiwat && newhiwat > 1) {
		a_info.hiwat = newhiwat;
		a_info.lowat = (a_info.hiwat * 3) / 4;
	}

	if (ioctl(audio_fd, AUDIO_SETINFO, &a_info) < 0) {
		fprintf(stderr, "audio_init: AUDIO_SETINFO\n");
		close(audio_fd);
		audio_fd = -1;
		return FAILURE;
	}

	return SUCCESS;
}

static BOOL
netbsdaudio_term(void)
{

	ioctl(audio_fd, AUDIO_DRAIN);
	close(audio_fd);

	return SUCCESS;
}

#endif	/* USE_NETBSDAUDIO */
